from twisted.python import failure
from twisted.internet import defer

from ooni.tasks import BaseTask, TaskWithTimeout
from ooni.managers import TaskManager
from ooni.backend_client import CollectorClient

class MockMeasurementFailOnce(BaseTask):
    def run(self):
        f = open('dummyTaskFailOnce.txt', 'w')
        f.write('fail')
        f.close()
        if self.failure >= 1:
            return defer.succeed(self)
        else:
            return defer.fail(failure.Failure)


class MockMeasurementManager(TaskManager):
    def __init__(self):
        self.successes = []
        TaskManager.__init__(self)

    def failed(self, failure, task):
        pass

    def succeeded(self, result, task):
        self.successes.append((result, task))


class MockReporter(object):
    def __init__(self):
        self.created = defer.Deferred()

    def writeReportEntry(self, entry):
        pass

    def createReport(self):
        self.created.callback(self)

    def finish(self):
        pass


class MockFailure(Exception):
    pass

# # from test_managers
mockFailure = failure.Failure(MockFailure('mock'))


class MockSuccessTask(BaseTask):
    def run(self):
        return defer.succeed(42)


class MockFailTask(BaseTask):
    def run(self):
        return defer.fail(mockFailure)


class MockFailOnceTask(BaseTask):
    def run(self):
        if self.failures >= 1:
            return defer.succeed(42)
        else:
            return defer.fail(mockFailure)


class MockSuccessTaskWithTimeout(TaskWithTimeout):
    def run(self):
        return defer.succeed(42)


class MockFailTaskThatTimesOut(TaskWithTimeout):
    def run(self):
        return defer.Deferred()


class MockTimeoutOnceTask(TaskWithTimeout):
    def run(self):
        if self.failures >= 1:
            return defer.succeed(42)
        else:
            return defer.Deferred()


class MockFailTaskWithTimeout(TaskWithTimeout):
    def run(self):
        return defer.fail(mockFailure)


class MockNetTest(object):
    def __init__(self):
        self.successes = []

    def succeeded(self, measurement):
        self.successes.append(measurement)


class MockMeasurement(TaskWithTimeout):
    def __init__(self, net_test):
        TaskWithTimeout.__init__(self)
        self.netTest = net_test

    def succeeded(self, result):
        return self.netTest.succeeded(42)


class MockSuccessMeasurement(MockMeasurement):
    def run(self):
        return defer.succeed(42)


class MockFailMeasurement(MockMeasurement):
    def run(self):
        return defer.fail(mockFailure)


class MockFailOnceMeasurement(MockMeasurement):
    def run(self):
        if self.failures >= 1:
            return defer.succeed(42)
        else:
            return defer.fail(mockFailure)


class MockDirector(object):
    def __init__(self):
        self.successes = []

    def measurementFailed(self, failure, measurement):
        pass

    def measurementSucceeded(self, measurement):
        self.successes.append(measurement)


## from test_reporter.py
class MockOReporter(object):
    def __init__(self):
        self.created = defer.Deferred()

    def writeReportEntry(self, entry):
        return defer.succeed(42)

    def finish(self):
        pass

    def createReport(self):
        from ooni.utils import log

        log.debug("Creating report with %s" % self)
        self.created.callback(self)


class MockOReporterThatFailsWrite(MockOReporter):
    def writeReportEntry(self, entry):
        raise MockFailure


class MockOReporterThatFailsOpen(MockOReporter):
    def createReport(self):
        raise MockFailure


class MockOReporterThatFailsWriteOnce(MockOReporter):
    def __init__(self):
        self.failure = 0
        MockOReporter.__init__(self)

    def writeReportEntry(self, entry):
        if self.failure >= 1:
            return defer.succeed(42)
        else:
            self.failure += 1
            raise MockFailure


class MockTaskManager(TaskManager):
    def __init__(self):
        self.successes = []
        TaskManager.__init__(self)

    def failed(self, failure, task):
        pass

    def succeeded(self, result, task):
        self.successes.append((result, task))


class MockBouncerClient(object):
    def __init__(self, *args, **kw):
        pass

    def lookupTestHelpers(self, required_test_helpers):
        ret = {
            'default': {
                'address': '127.0.0.1',
                'collector': 'httpo://thirteenchars123.onion'
            }
        }
        for required_test_helper in required_test_helpers:
            ret[required_test_helper] = {
                'address': '127.0.0.1',
                'collector': 'httpo://thirteenchars123.onion'
            }
        return defer.succeed(ret)

    def lookupTestCollector(self, net_tests):
        ret = {
            'net-tests': [
            ]
        }
        for net_test in net_tests:
            test_helpers ={}
            for test_helper in net_test['test-helpers']:
                test_helpers[test_helper] = '127.0.0.1'

            ret['net-tests'].append({
                'name': net_test['name'],
                'version': net_test['version'],
                'input-hashes': net_test['input-hashes'],
                'collector': 'httpo://thirteenchars123.onion',
                'collector-alternate': [
                    {'type': 'https', 'address': 'https://collector.ooni.io'},
                    {'type': 'http', 'address': 'http://collector.ooni.io'},
                    {
                        'type': 'cloudfront',
                        'address': 'https://address.cloudfront.net',
                        'front': 'front.cloudfront.net'
                    },
                ],
                'test-helpers': test_helpers
            })
        return defer.succeed(ret)


class MockCollectorClient(CollectorClient):
    def isSupported(self):
        return True

    def isReachable(self):
        return defer.succeed(True)



import sys
import time
import random
from twisted.internet import fdesc
from twisted.internet import reactor
from twisted.internet import defer, abstract
from scapy.config import conf
from scapy.all import RandShort, IP, IPerror, ICMP, ICMPerror, TCP, TCPerror, UDP, UDPerror

from ooni.errors import ProtocolNotRegistered, ProtocolAlreadyRegistered, LibraryNotInstalledError

from ooni.utils import log

from ooni.utils.net import getDefaultIface, getAddresses
from ooni.settings import config


def pcapdnet_installed():
    """
    Checks to see if libdnet or libpcap are installed and set the according
    variables.

    Returns:

        True
            if pypcap and libdnet are installed

        False
            if one of the two is absent
    """
    # In debian libdnet is called dumbnet instead of dnet, but scapy is
    # expecting "dnet" so we try and import it under such name.
    try:
        import dumbnet

        sys.modules['dnet'] = dumbnet
    except ImportError:
        pass

    try:
        conf.use_pcap = True
        conf.use_dnet = True
        from scapy.arch import pcapdnet

        config.pcap_dnet = True

    except ImportError as e:
        log.err(e.message + ". Pypcap or dnet are not properly installed. Certain tests may not work.")
        config.pcap_dnet = False
        conf.use_pcap = False
        conf.use_dnet = False

    # This is required for unix systems that are different than linux (OSX for
    # example) since scapy explicitly wants pcap and libdnet installed for it
    # to work.
    try:
        from scapy.arch import pcapdnet
    except ImportError:
        log.err("Your platform requires having libdnet and libpcap installed.")
        raise LibraryNotInstalledError

    return config.pcap_dnet


if pcapdnet_installed():
    from scapy.all import PcapWriter

else:

    class DummyPcapWriter:
        def __init__(self, pcap_filename, *arg, **kw):
            log.err("Initializing DummyPcapWriter. We will not actually write to a pcapfile")

        @staticmethod
        def write(self):
            pass

    PcapWriter = DummyPcapWriter

from scapy.all import Gen, SetGen, MTU


class ScapyFactory(abstract.FileDescriptor):
    """
    Inspired by muxTCP scapyLink:
    https://github.com/enki/muXTCP/blob/master/scapyLink.py
    """

    def __init__(self, interface, super_socket=None, timeout=5):

        abstract.FileDescriptor.__init__(self, reactor)
        if interface == 'auto':
            interface = getDefaultIface()
        if not super_socket and sys.platform == 'darwin':
            super_socket = conf.L3socket(iface=interface, promisc=True, filter='')
        elif not super_socket:
            super_socket = conf.L3socket(iface=interface)

        self.protocols = []
        fdesc._setCloseOnExec(super_socket.ins.fileno())
        self.super_socket = super_socket

    def writeSomeData(self, data):
        """
        XXX we actually want to use this, but this requires overriding doWrite
        or writeSequence.
        """
        pass

    def send(self, packet):
        """
        Write a scapy packet to the wire.
        """
        return self.super_socket.send(packet)

    def fileno(self):
        return self.super_socket.ins.fileno()

    def doRead(self):
        packet = self.super_socket.recv(MTU)
        if packet:
            for protocol in self.protocols:
                protocol.packetReceived(packet)

    def registerProtocol(self, protocol):
        if not self.connected:
            self.startReading()

        if protocol not in self.protocols:
            protocol.factory = self
            self.protocols.append(protocol)
        else:
            raise ProtocolAlreadyRegistered

    def unRegisterProtocol(self, protocol):
        if protocol in self.protocols:
            self.protocols.remove(protocol)
            if len(self.protocols) == 0:
                self.loseConnection()
        else:
            raise ProtocolNotRegistered

class ScapyProtocol(object):
    factory = None

    def packetReceived(self, packet):
        """
        When you register a protocol, this method will be called with argument
        the packet it received.

        Every protocol that is registered will have this method called.
        """
        raise NotImplementedError


class ScapySender(ScapyProtocol):
    timeout = 5

    # This deferred will fire when we have finished sending a receiving packets.
    # Should we look for multiple answers for the same sent packet?
    multi = False

    # When 0 we stop when all the packets we have sent have received an
    # answer
    expected_answers = 0

    def processPacket(self, packet):
        """
        Hook useful for processing packets as they come in.
        """

    def processAnswer(self, packet, answer_hr):
        log.debug("Got a packet from %s" % packet.src)
        log.debug("%s" % self.__hash__)
        for i in range(len(answer_hr)):
            if packet.answers(answer_hr[i]):
                self.answered_packets.append((answer_hr[i], packet))
                if not self.multi:
                    del (answer_hr[i])
                break

        if len(self.answered_packets) == len(self.sent_packets):
            log.debug("All of our questions have been answered.")
            self.stopSending()
            return

        if self.expected_answers and self.expected_answers == len(self.answered_packets):
            log.debug("Got the number of expected answers")
            self.stopSending()

    def packetReceived(self, packet):
        if self.timeout and time.time() - self._start_time > self.timeout:
            self.stopSending()
        if packet:
            self.processPacket(packet)
            # A string that has the same value for the request than for the
            # response.
            hr = packet.hashret()
            if hr in self.hr_sent_packets:
                answer_hr = self.hr_sent_packets[hr]
                self.processAnswer(packet, answer_hr)

    def stopSending(self):
        result = (self.answered_packets, self.sent_packets)
        self.d.callback(result)
        self.factory.unRegisterProtocol(self)

    def sendPackets(self, packets):
        if not isinstance(packets, Gen):
            packets = SetGen(packets)
        for packet in packets:
            hashret = packet.hashret()
            if hashret in self.hr_sent_packets:
                self.hr_sent_packets[hashret].append(packet)
            else:
                self.hr_sent_packets[hashret] = [packet]
            self.sent_packets.append(packet)
            self.factory.send(packet)

    def startSending(self, packets):
        # This dict is used to store the unique hashes that allow scapy to
        # match up request with answer
        self.hr_sent_packets = {}

        # These are the packets we have received as answer to the ones we sent
        self.answered_packets = []

        # These are the packets we send
        self.sent_packets = []

        self._start_time = time.time()
        self.d = defer.Deferred()
        self.sendPackets(packets)
        return self.d


class ScapySniffer(ScapyProtocol):
    def __init__(self, pcap_filename, *arg, **kw):
        self.pcapwriter = PcapWriter(pcap_filename, *arg, **kw)

    def packetReceived(self, packet):
        self.pcapwriter.write(packet)

    def close(self):
        self.pcapwriter.close()


class ParasiticTraceroute(ScapyProtocol):
    def __init__(self):
        self.numHosts = 7
        self.rate = 15
        self.hosts = {}
        self.ttl_max = 15
        self.ttl_min = 1
        self.sent_packets = []
        self.received_packets = []
        self.matched_packets = {}
        self.addresses = [str(x) for x in getAddresses()]

    def sendPacket(self, packet):
        self.factory.send(packet)
        self.sent_packets.append(packet)
        log.debug("Sent packet to %s with ttl %d" % (packet.dst, packet.ttl))

    def packetReceived(self, packet):
        try:
            packet[IP]
        except IndexError:
            return

        # Add TTL Expired responses.
        if isinstance(packet.getlayer(3), TCPerror):
            self.received_packets.append(packet)
            # Live traceroute?
            log.debug("%s replied with icmp-ttl-exceeded for %s" % (packet.src, packet[IPerror].dst))
            return
        elif packet.dst in self.hosts:
            if random.randint(1, 100) > self.rate:
                # Don't send a packet this time
                return
            try:
                packet[IP].ttl = self.hosts[packet.dst]['ttl'].pop()
                del packet.chksum  # XXX Why is this incorrect?
                self.sendPacket(packet)
                k = (packet.id, packet[TCP].sport, packet[TCP].dport, packet[TCP].seq)
                self.matched_packets[k] = {'ttl': packet.ttl}
                return
            except IndexError:
                return

        def maxttl(packet=None):
            if packet:
                return min(self.ttl_max, *map(lambda x: x - packet.ttl, [64, 128, 256])) - 1
            else:
                return self.ttl_max

        def genttl(packet=None):
            ttl = range(self.ttl_min, maxttl(packet))
            random.shuffle(ttl)
            return ttl

        if len(self.hosts) < self.numHosts:
            if packet.dst not in self.hosts \
                    and packet.dst not in self.addresses \
                    and isinstance(packet.getlayer(1), TCP):

                self.hosts[packet.dst] = {'ttl': genttl()}
                log.debug("Tracing to %s" % packet.dst)
                return
            if packet.src not in self.hosts \
                    and packet.src not in self.addresses \
                    and isinstance(packet.getlayer(1), TCP):
                self.hosts[packet.src] = {'ttl': genttl(packet),
                                          'ttl_max': maxttl(packet)}
                log.debug("Tracing to %s" % packet.src)
                return

        if packet.src in self.hosts and not 'ttl_max' in self.hosts[packet.src]:
            self.hosts[packet.src]['ttl_max'] = ttl_max = maxttl(packet)
            log.debug("set ttl_max to %d for host %s" % (ttl_max, packet.src))
            ttl = []
            for t in self.hosts[packet.src]['ttl']:
                if t < ttl_max:
                    ttl.append(t)
            self.hosts[packet.src]['ttl'] = ttl
            return

    def stopListening(self):
        self.factory.unRegisterProtocol(self)


class MPTraceroute(ScapyProtocol):
    dst_ports = [0, 22, 23, 53, 80, 123, 443, 8080, 65535]
    ttl_min = 1
    ttl_max = 30

    def __init__(self):
        self.sent_packets = []
        self._recvbuf = []
        self.received_packets = {}
        self.matched_packets = {}
        self.hosts = []
        self.interval = 0.2
        self.timeout = ((self.ttl_max - self.ttl_min) * len(self.dst_ports) * self.interval) + 5
        self.numPackets = 1

    def ICMPTraceroute(self, host):
        if host not in self.hosts:
            self.hosts.append(host)

        d = defer.Deferred()
        reactor.callLater(self.timeout, d.callback, self)

        self.sendPackets(IP(dst=host, ttl=(self.ttl_min, self.ttl_max), id=RandShort()) / ICMP(id=RandShort()))
        return d

    def UDPTraceroute(self, host):
        if host not in self.hosts:
            self.hosts.append(host)

        d = defer.Deferred()
        reactor.callLater(self.timeout, d.callback, self)

        for dst_port in self.dst_ports:
            self.sendPackets(
                IP(dst=host, ttl=(self.ttl_min, self.ttl_max), id=RandShort()) / UDP(dport=dst_port, sport=RandShort()))
        return d

    def TCPTraceroute(self, host):
        if host not in self.hosts:
            self.hosts.append(host)

        d = defer.Deferred()
        reactor.callLater(self.timeout, d.callback, self)

        for dst_port in self.dst_ports:
            self.sendPackets(
                IP(dst=host, ttl=(self.ttl_min, self.ttl_max), id=RandShort()) / TCP(flags=2L, dport=dst_port,
                                                                                     sport=RandShort(),
                                                                                     seq=RandShort()))
        return d

    @defer.inlineCallbacks
    def sendPackets(self, packets):
        def sleep(seconds):
            d = defer.Deferred()
            reactor.callLater(seconds, d.callback, seconds)
            return d

        if not isinstance(packets, Gen):
            packets = SetGen(packets)

        for packet in packets:
            for i in xrange(self.numPackets):
                self.sent_packets.append(packet)
                self.factory.super_socket.send(packet)
                yield sleep(self.interval)

    def matchResponses(self):
        def addToReceivedPackets(key, packet):
            """
            Add a packet into the received packets dictionary,
            typically the key is a tuple of packet fields used
            to correlate sent packets with received packets.
            """

            # Initialize or append to the lists of packets
            # with the same key
            if key in self.received_packets:
                self.received_packets[key].append(packet)
            else:
                self.received_packets[key] = [packet]

        def matchResponse(k, p):
            if k in self.received_packets:
                if p in self.matched_packets:
                    log.debug("Matched sent packet to more than one response!")
                    self.matched_packets[p].extend(self.received_packets[k])
                else:
                    self.matched_packets[p] = self.received_packets[k]
                log.debug("Packet %s matched %s" % ([p], self.received_packets[k]))
                return 1
            return 0

        for p in self._recvbuf:
            l = p.getlayer(2)
            if isinstance(l, IPerror):
                l = p.getlayer(3)
                if isinstance(l, ICMPerror):
                    addToReceivedPackets(('icmp', l.id), p)
                elif isinstance(l, TCPerror):
                    addToReceivedPackets(('tcp', l.dport, l.sport), p)
                elif isinstance(l, UDPerror):
                    addToReceivedPackets(('udp', l.dport, l.sport), p)
            elif hasattr(p, 'src') and p.src in self.hosts:
                l = p.getlayer(1)
                if isinstance(l, ICMP):
                    addToReceivedPackets(('icmp', l.id), p)
                elif isinstance(l, TCP):
                    addToReceivedPackets(('tcp', l.ack - 1, l.dport, l.sport), p)
                elif isinstance(l, UDP):
                    addToReceivedPackets(('udp', l.dport, l.sport), p)

        for p in self.sent_packets:
            # for each sent packet, find corresponding
            # received packets
            l = p.getlayer(1)
            i = 0
            if isinstance(l, ICMP):
                i += matchResponse(('icmp', p.id), p)  # match by ipid
                i += matchResponse(('icmp', l.id), p)  # match by icmpid
            if isinstance(l, TCP):
                i += matchResponse(('tcp', l.dport, l.sport), p)  # match by s|dport
                i += matchResponse(('tcp', l.seq, l.sport, l.dport), p)
            if isinstance(l, UDP):
                i += matchResponse(('udp', l.dport, l.sport), p)
                i += matchResponse(('udp', l.sport, l.dport), p)
            if i == 0:
                log.debug("No response for packet %s" % [p])

        del self._recvbuf

    def packetReceived(self, packet):
        l = packet.getlayer(1)
        if not l:
            return
        elif isinstance(l, ICMP) or isinstance(l, UDP) or isinstance(l, TCP):
            self._recvbuf.append(packet)

    def stopListening(self):
        self.factory.unRegisterProtocol(self)



import tempfile
import os
import sys

from twisted.internet import defer, reactor
from twisted.internet.error import ProcessExitedAlready
from twisted.python import usage

from ooni.utils import log, net
from ooni.templates import process, httpt


class UsageOptions(usage.Options):
    optParameters = [
        ['psiphonpath', 'p', None, 'Specify psiphon python client path.'],
        ['url', 'u', net.GOOGLE_HUMANS[0],
            'Specify the URL to fetch over psiphon (default: http://www.google.com/humans.txt).'],
        ['expected-body', 'e', net.GOOGLE_HUMANS[1],
            'Specify the beginning of the expected body in the response (default: ' + net.GOOGLE_HUMANS[1] + ').']
    ]

class PsiphonTest(httpt.HTTPTest,  process.ProcessTest):

    """
    This class tests Psiphon python client

    test_psiphon:
      Starts a Psiphon, check if it bootstraps successfully
      (print a line in stdout).
      Then, perform an HTTP request using the proxy
    """

    name = "Psiphon Test"
    description = ("Bootstraps Psiphon and "
                   "does a HTTP GET for the specified URL.")
    author = "juga"
    version = "0.1.0"
    timeout = 120
    usageOptions = UsageOptions

    def _setUp(self):
        self.localOptions['socksproxy'] = '127.0.0.1:1080'
        super(PsiphonTest, self)._setUp()

    def setUp(self):
        log.debug('PsiphonTest.setUp')

        self.report['bootstrapped_success'] = None
        self.report['request_success'] = None
        self.report['psiphon_found'] = None
        self.report['default_configuration'] = True

        self.bootstrapped = defer.Deferred()
        self.url = self.localOptions['url']

        if self.localOptions['url'] != net.GOOGLE_HUMANS[0]:
            self.report['default_configuration'] = False

        if self.localOptions['expected-body'] != net.GOOGLE_HUMANS[1]:
            self.report['default_configuration'] = False

        if self.localOptions['psiphonpath']:
            self.psiphonpath = self.localOptions['psiphonpath']
        else:
            # Psiphon is not installable and to run it manually, it has to be
            # run from the psiphon directory, so it wouldn't make sense to
            # install it in the PATH. For now, we assume that Psiphon sources
            # are in the user's home directory.
            from os import path, getenv
            self.psiphonpath = path.join(
                getenv('HOME'), 'psiphon-circumvention-system/pyclient/pyclient')
            log.debug('psiphon path: %s' % self.psiphonpath)

    def createCommand(self):
        # psi_client.py can not be run directly because the paths in the
        # code are relative, so it'll fail to execute from this test
        x = """
from psi_client import connect
connect(False)
"""
        f = tempfile.NamedTemporaryFile(delete=False)
        f.write(x)
        f.close()
        self.command = [sys.executable, f.name]
        log.debug('command: %s' % ' '.join(self.command))

    def handleRead(self, stdout, stderr):
        if 'Press Ctrl-C to terminate.' in self.processDirector.stdout:
            if not self.bootstrapped.called:
                # here the text 'Press Ctrl-C to terminate.' has been found
                # and it was to call doRequest
                self.report['bootstrapped_success'] = True
                log.debug("PsiphonTest: calling bootstrapped.callback")
                self.bootstrapped.callback(None)

    def test_psiphon(self):
        log.debug('PsiphonTest.test_psiphon')
        self.createCommand()
        if not os.path.exists(self.psiphonpath):
            log.err('psiphon path does not exists, is it installed?')
            self.report['psiphon_found'] = False
            log.debug("Adding %s to report" % self.report)
            # XXX: the original code written by juga0 readed
            #     > return defer.succeed(None)
            # but this caused `ooniprobe -ng` to hang forever, so I
            # rewrote the code to return a deferred and simulate calling
            # its callback method, to trigger an event.
            #     -sbs
            reactor.callLater(0.0, self.bootstrapped.callback, None)
            return self.bootstrapped

        self.report['psiphon_found'] = True
        log.debug("Adding %s to report" % self.report)

        # Using pty to see output lines as soon as they get wrotten in the
        # buffer, otherwise the test might not see lines until the buffer is
        # full with some block size and therefore the test would
        # terminate with error
        finished = self.run(self.command,
                            env=dict(PYTHONPATH=self.psiphonpath),
                            path=self.psiphonpath,
                            usePTY=1)
        # here psiphon command has been run, and if it finds the text
        # 'Press Ctrl-C to terminate' in handleRead it will write to the
        # report self.report['bootstrapped_success'] = True
        self.report['bootstrapped_success'] = False

        def callDoRequest(_):
            log.debug("PsiphonTest.callDoRequest: %r" %(_,))
            d = self.doRequest(self.url)
            def addSuccessToReport(res):
                log.debug("PsiphonTest.callDoRequest.addSuccessToReport")
                if res.body.startswith(self.localOptions['expected-body']):
                    self.report['request_success'] = True
                else:
                    self.report['request_success'] = False

                return res
            d.addCallback(addSuccessToReport)
            def addFailureToReport(res):
                log.debug("PsiphonTest.callDoRequest.addFailureToReport. res=%r" % (res,))
                self.report['request_success'] = False
                return res
            d.addErrback(addFailureToReport)
            return d
        self.bootstrapped.addCallback(callDoRequest)

        def cleanup(_):
            log.debug('PsiphonTest:cleanup')
            try:
                self.processDirector.transport.signalProcess('INT')
            except ProcessExitedAlready:
                pass
            os.remove(self.command[1])
            return finished

        self.bootstrapped.addBoth(cleanup)
        return self.bootstrapped



import os

from twisted.internet import defer
from twisted.trial import unittest

from hashlib import sha256
from ooni import errors
from ooni.deck import InputFile, Deck, nettest_to_path
from ooni.tests.bases import ConfigTestCase
from ooni.tests.mocks import MockBouncerClient, MockCollectorClient

FAKE_BOUNCER_ADDRESS = "httpo://thirteenchars123.onion"

net_test_string = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptions(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

class DummyTestCase(NetTestCase):

    usageOptions = UsageOptions
    requiredTestHelpers = {'spam': 'test-helper-typeA'}

    def test_a(self):
        self.report['bar'] = 'bar'

    def test_b(self):
        self.report['foo'] = 'foo'
"""


class BaseTestCase(unittest.TestCase):
    def setUp(self):
        self.filename = ""
        self.cwd = os.getcwd()
        self.dummy_deck_content = """- options:
            collector: null
            annotations: null
            help: 0
            logfile: null
            no-default-reporter: 0
            parallelism: null
            pcapfile: null
            reportfile: null
            resume: 0
            subargs: []
            test_file: manipulation/http_invalid_request_line
            testdeck: null
"""
        self.dummy_deck_content_with_many_tests = """- options:
            collector: null
            help: 0
            logfile: null
            no-default-reporter: 0
            parallelism: null
            pcapfile: null
            reportfile: null
            resume: 0
            subargs: [-b, "1.1.1.1"]
            test_file: manipulation/http_invalid_request_line
            testdeck: null
- options:
            collector: null
            help: 0
            logfile: null
            no-default-reporter: 0
            parallelism: null
            pcapfile: null
            reportfile: null
            resume: 0
            subargs: [-b, "2.2.2.2"]
            test_file: manipulation/http_invalid_request_line
            testdeck: null
"""
        super(BaseTestCase, self).setUp()


class TestInputFile(BaseTestCase):
    def tearDown(self):
        if self.filename != "":
            os.remove(self.filename)

    def test_file_cached(self):
        self.filename = file_hash = sha256(self.dummy_deck_content).hexdigest()
        input_file = InputFile(file_hash, base_path='.')
        with open(file_hash, 'w+') as f:
            f.write(self.dummy_deck_content)
        assert input_file.fileCached

    def test_file_invalid_hash(self):
        self.filename = invalid_hash = 'a' * 64
        with open(invalid_hash, 'w+') as f:
            f.write("b" * 100)
        input_file = InputFile(invalid_hash, base_path='.')
        self.assertRaises(AssertionError, input_file.verify)

    def test_save_descriptor(self):
        descriptor = {
            'name': 'spam',
            'id': 'spam',
            'version': 'spam',
            'author': 'spam',
            'date': 'spam',
            'description': 'spam'
        }
        file_id = 'a' * 64
        self.filename = file_id + '.desc'
        input_file = InputFile(file_id, base_path='.')
        input_file.load(descriptor)
        input_file.save()
        assert os.path.isfile(self.filename)

        assert input_file.descriptorCached


class TestDeck(BaseTestCase, ConfigTestCase):
    def setUp(self):
        super(TestDeck, self).setUp()
        deck_hash = sha256(self.dummy_deck_content).hexdigest()
        self.deck_file = os.path.join(self.cwd, deck_hash)
        with open(self.deck_file, 'w+') as f:
            f.write(self.dummy_deck_content)
        with open(os.path.join(self.cwd, 'dummy_test.py'), 'w+') as f:
            f.write(net_test_string)

    def tearDown(self):
        os.remove(os.path.join(self.cwd, 'dummy_test.py'))
        os.remove(self.deck_file)
        if self.filename != "":
            os.remove(self.filename)
        super(TestDeck, self).tearDown()

    def test_open_deck(self):
        deck = Deck(bouncer=FAKE_BOUNCER_ADDRESS,
                    decks_directory=".")
        deck.loadDeck(self.deck_file)
        assert len(deck.netTestLoaders) == 1

    def test_load_deck_with_global_options(self):
        global_options = {
            "annotations": {"spam": "ham"},
            "collector": "httpo://thirteenchars123.onion"
        }
        deck = Deck(bouncer=FAKE_BOUNCER_ADDRESS,
                    decks_directory=".")
        deck.loadDeck(self.deck_file,
                      global_options=global_options)
        self.assertEqual(
            deck.netTestLoaders[0].annotations,
            global_options['annotations']
        )
        self.assertEqual(
            deck.netTestLoaders[0].collector.base_address,
            global_options['collector'].replace("httpo://", "http://")
        )

    def test_save_deck_descriptor(self):
        deck = Deck(bouncer=FAKE_BOUNCER_ADDRESS,
                    decks_directory=".")
        deck.loadDeck(self.deck_file)
        deck.load({'name': 'spam',
                   'id': 'spam',
                   'version': 'spam',
                   'author': 'spam',
                   'date': 'spam',
                   'description': 'spam'
                   })
        deck.save()
        self.filename = self.deck_file + ".desc"
        deck.verify()

    @defer.inlineCallbacks
    def test_lookup_test_helpers_and_collector(self):
        deck = Deck(bouncer=FAKE_BOUNCER_ADDRESS,
                    decks_directory=".")
        deck.bouncer = MockBouncerClient(FAKE_BOUNCER_ADDRESS)
        deck._BouncerClient = MockBouncerClient
        deck._CollectorClient = MockCollectorClient
        deck.loadDeck(self.deck_file)

        self.assertEqual(len(deck.netTestLoaders[0].missingTestHelpers), 1)

        yield deck.lookupCollectorAndTestHelpers()

        self.assertEqual(deck.netTestLoaders[0].collector.settings['address'],
                         'httpo://thirteenchars123.onion')

        self.assertEqual(deck.netTestLoaders[0].localOptions['backend'],
                         '127.0.0.1')


    def test_deck_with_many_tests(self):
        os.remove(self.deck_file)
        deck_hash = sha256(self.dummy_deck_content_with_many_tests).hexdigest()
        self.deck_file = os.path.join(self.cwd, deck_hash)
        with open(self.deck_file, 'w+') as f:
            f.write(self.dummy_deck_content_with_many_tests)
        deck = Deck(decks_directory=".")
        deck.loadDeck(self.deck_file)

        self.assertEqual(
            deck.netTestLoaders[0].localOptions['backend'],
            '1.1.1.1'
        )
        self.assertEqual(
            deck.netTestLoaders[1].localOptions['backend'],
            '2.2.2.2'
        )

    def test_nettest_to_path(self):
        path_a = nettest_to_path("blocking/http_requests")
        path_b = nettest_to_path("http_requests")
        self.assertEqual(path_a, path_b)
        self.assertRaises(errors.NetTestNotFound,
                          nettest_to_path,
                          "invalid_test")

    @defer.inlineCallbacks
    def test_lookup_test_helpers_and_collector_cloudfront(self):
        self.config.advanced.preferred_backend = "cloudfront"
        deck = Deck(bouncer=FAKE_BOUNCER_ADDRESS,
                    decks_directory=".")
        deck.bouncer = MockBouncerClient(FAKE_BOUNCER_ADDRESS)
        deck._BouncerClient = MockBouncerClient
        deck._CollectorClient = MockCollectorClient
        deck.loadDeck(self.deck_file)

        self.assertEqual(len(deck.netTestLoaders[0].missingTestHelpers), 1)

        yield deck.lookupCollectorAndTestHelpers()

        self.assertEqual(
            deck.netTestLoaders[0].collector.settings['address'],
            'https://address.cloudfront.net'
        )
        self.assertEqual(
            deck.netTestLoaders[0].collector.settings['front'],
            'front.cloudfront.net'
        )

        self.assertEqual(
            deck.netTestLoaders[0].localOptions['backend'],
            '127.0.0.1'
        )


    @defer.inlineCallbacks
    def test_lookup_test_helpers_and_collector_https(self):
        self.config.advanced.preferred_backend = "https"
        deck = Deck(bouncer=FAKE_BOUNCER_ADDRESS,
                    decks_directory=".")
        deck.bouncer = MockBouncerClient(FAKE_BOUNCER_ADDRESS)
        deck._BouncerClient = MockBouncerClient
        deck._CollectorClient = MockCollectorClient
        deck.loadDeck(self.deck_file)

        self.assertEqual(len(deck.netTestLoaders[0].missingTestHelpers), 1)

        yield deck.lookupCollectorAndTestHelpers()

        self.assertEqual(
            deck.netTestLoaders[0].collector.settings['address'],
            'https://collector.ooni.io'
        )

        self.assertEqual(
            deck.netTestLoaders[0].localOptions['backend'],
            '127.0.0.1'
        )



import os
import yaml
import json
import time
from mock import MagicMock

from twisted.internet import defer
from twisted.trial import unittest

from ooni import errors as e
from ooni.tests.mocks import MockCollectorClient
from ooni.reporter import YAMLReporter, OONIBReporter, OONIBReportLog



class MockTest(object):
    _start_time = time.time()
    report = {'report_content': 'ham'}
    input = 'spam'


test_details = {
    'test_name': 'spam',
    'test_version': '1.0',
    'software_name': 'spam',
    'software_version': '1.0',
    'input_hashes': [],
    'probe_asn': 'AS0',
    'probe_cc': 'ZZ',
    'test_start_time': '2016-01-01 22:33:11',
    'data_format_version': '0.2.0'
}

oonib_new_report_message = {
    'report_id': "20140129T202038Z_AS0_" + "A" * 50,
    'backend_version': "1.0",
    'supported_formats': ["yaml", "json"]
}

# This is used for testing legacy collectors
oonib_new_report_yaml_message = {
    'report_id': "20140129T202038Z_AS0_" + "A" * 50,
    'backend_version': "1.0"
}


oonib_generic_error_message = {
    'error': 'generic-error'
}


class TestYAMLReporter(unittest.TestCase):

    def setUp(self):
        self.filename = ""

    def tearDown(self):
        if self.filename != "":
            os.remove(self.filename)

    def test_write_report(self):
        y_reporter = YAMLReporter(test_details, 'dummy-report.yaml')
        y_reporter.createReport()
        with open(y_reporter.report_path) as f:
            self.filename = y_reporter.report_path
            report_entries = yaml.safe_load_all(f)
            # Check for keys in header
            entry = report_entries.next()
            assert all(x in entry for x in ['test_name', 'test_version'])


class TestOONIBReporter(unittest.TestCase):

    def setUp(self):
        self.mock_response = {}

        def mockRequest(method, urn, genReceiver, *args, **kw):
            receiver = genReceiver(None, None)
            return defer.maybeDeferred(receiver.body_processor,
                                       json.dumps(self.mock_response))

        mock_collector_client = MockCollectorClient('http://example.com')
        mock_collector_client._request = mockRequest

        self.oonib_reporter = OONIBReporter(
            test_details,
            mock_collector_client
        )

    @defer.inlineCallbacks
    def test_create_report(self):
        self.mock_response = oonib_new_report_message
        yield self.oonib_reporter.createReport()
        self.assertEqual(self.oonib_reporter.reportId,
                         oonib_new_report_message['report_id'])

    @defer.inlineCallbacks
    def test_create_report_failure(self):
        self.mock_response = oonib_generic_error_message
        yield self.assertFailure(self.oonib_reporter.createReport(),
                                 e.OONIBReportCreationError)

    @defer.inlineCallbacks
    def test_write_report_entry(self):
        self.mock_response = oonib_new_report_message
        yield self.oonib_reporter.createReport()
        req = {'content': 'something'}
        yield self.oonib_reporter.writeReportEntry(req)

    @defer.inlineCallbacks
    def test_write_report_entry_in_yaml(self):
        self.mock_response = oonib_new_report_yaml_message
        yield self.oonib_reporter.createReport()
        req = {'content': 'something'}
        yield self.oonib_reporter.writeReportEntry(req)

class TestOONIBReportLog(unittest.TestCase):

    def setUp(self):
        self.report_log = OONIBReportLog('report_log')
        self.report_log.create_report_log()

    def tearDown(self):
        os.remove(self.report_log.file_name)

    @defer.inlineCallbacks
    def test_report_created(self):
        yield self.report_log.created("path_to_my_report.yaml",
                                             'httpo://foo.onion',
                                             'someid')
        with open(self.report_log.file_name) as f:
            report = yaml.safe_load(f)
            assert "path_to_my_report.yaml" in report

    @defer.inlineCallbacks
    def test_concurrent_edit(self):
        d1 = self.report_log.created("path_to_my_report1.yaml",
                                            'httpo://foo.onion',
                                            'someid1')
        d2 = self.report_log.created("path_to_my_report2.yaml",
                                            'httpo://foo.onion',
                                            'someid2')
        yield defer.DeferredList([d1, d2])
        with open(self.report_log.file_name) as f:
            report = yaml.safe_load(f)
            assert "path_to_my_report1.yaml" in report
            assert "path_to_my_report2.yaml" in report

    @defer.inlineCallbacks
    def test_report_closed(self):
        yield self.report_log.created("path_to_my_report.yaml",
                                             'httpo://foo.onion',
                                             'someid')
        yield self.report_log.closed("path_to_my_report.yaml")

        with open(self.report_log.file_name) as f:
            report = yaml.safe_load(f)
            assert "path_to_my_report.yaml" not in report

    @defer.inlineCallbacks
    def test_report_creation_failed(self):
        yield self.report_log.creation_failed("path_to_my_report.yaml",
                                                     'httpo://foo.onion')
        with open(self.report_log.file_name) as f:
            report = yaml.safe_load(f)
        assert "path_to_my_report.yaml" in report
        assert report["path_to_my_report.yaml"]["status"] == "creation-failed"

    @defer.inlineCallbacks
    def test_list_reports(self):
        yield self.report_log.creation_failed("failed_report.yaml",
                                              'httpo://foo.onion')
        yield self.report_log.created("created_report.yaml",
                                      'httpo://foo.onion', 'XXXX')

        assert len(self.report_log.reports_in_progress) == 1
        assert len(self.report_log.reports_incomplete) == 0
        assert len(self.report_log.reports_to_upload) == 1



# -*- encoding: utf-8 -*-
#
# :authors: Arturo Filastò
# :licence: see LICENSE

from twisted.python import usage

from ooni.templates import httpt

class UsageOptions(usage.Options):
    optParameters = [['backend', 'b', 'http://127.0.0.1:57001',
                        'URL of the test backend to use']]

class HTTPKeywordFiltering(httpt.HTTPTest):
    """
    This test involves performing HTTP requests containing to be tested for
    censorship keywords.

    It does not detect censorship on the client, but just logs the response from the 
    HTTP backend server.
    """
    name = "HTTP Keyword Filtering"
    author = "Arturo Filastò"
    version = "0.1.1"

    inputFile = ['file', 'f', None, 'List of keywords to use for censorship testing']

    usageOptions = UsageOptions
    requiresTor = False
    requiresRoot = False

    requiredOptions = ['backend']

    def test_get(self):
        """
        Perform a HTTP GET request to the backend containing the keyword to be
        tested inside of the request body.
        """
        return self.doRequest(self.localOptions['backend'], method="GET", body=self.input)

    def test_post(self):
        """
        Perform a HTTP POST request to the backend containing the keyword to be
        tested inside of the request body.
        """
        return self.doRequest(self.localOptions['backend'], method="POST", body=self.input)




# -*- encoding: utf-8 -*-
#
# :authors: Arturo Filastò
# :licence: see LICENSE

from twisted.python import usage

from ooni.templates import scapyt

from scapy.layers.inet import TCP, IP
from scapy.volatile import RandShort


class UsageOptions(usage.Options):
    optParameters = [
        ['backend', 'b', '127.0.0.1:57002', 'Test backend running TCP echo'],
        ['timeout', 't', 5, 'Timeout after which to give up waiting for RST packets']
    ]


class KeywordFiltering(scapyt.BaseScapyTest):
    name = "Keyword Filtering detection based on RST packets"
    author = "Arturo Filastò"
    version = "0.2"

    usageOptions = UsageOptions

    inputFile = ['file', 'f', None,
                 'List of keywords to use for censorship testing']
    requiresRoot = True
    requiresTor = False

    def test_tcp_keyword_filtering(self):
        """
        Places the keyword to be tested in the payload of a TCP packet.
        XXX need to implement bisection method for enumerating keywords.
            though this should not be an issue since we are testing all 
            the keywords in parallel.
        """
        backend_ip, backend_port = self.localOptions['backend'].split(':')
        timeout = int(self.localOptions['timeout'])
        keyword_to_test = str(self.input)
        packets = IP(dst=backend_ip, id=RandShort()) / TCP(sport=4000, dport=int(backend_port)) / keyword_to_test
        d = self.sr(packets, timeout=timeout)

        @d.addCallback
        def finished(packets):
            answered, unanswered = packets
            self.report['rst_packets'] = []
            for snd, rcv in answered:
                # The received packet has the RST flag
                if rcv[TCP].flags == 4:
                    self.report['rst_packets'].append(rcv)
        return d




import os
import yaml
from tempfile import mkstemp

from twisted.trial import unittest
from twisted.internet import defer, reactor
from twisted.python.usage import UsageError

from ooni.settings import config
from ooni.errors import MissingRequiredOption, OONIUsageError, IncoherentOptions
from ooni.nettest import NetTest, NetTestLoader

from ooni.director import Director

from ooni.tests.bases import ConfigTestCase

net_test_string = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptions(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

class DummyTestCase(NetTestCase):

    usageOptions = UsageOptions

    def test_a(self):
        self.report['bar'] = 'bar'

    def test_b(self):
        self.report['foo'] = 'foo'
"""

double_net_test_string = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptions(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

class DummyTestCaseA(NetTestCase):

    usageOptions = UsageOptions

    def test_a(self):
        self.report['bar'] = 'bar'


class DummyTestCaseB(NetTestCase):

    usageOptions = UsageOptions

    def test_b(self):
        self.report['foo'] = 'foo'
"""

double_different_options_net_test_string = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptionsA(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

class UsageOptionsB(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

class DummyTestCaseA(NetTestCase):

    usageOptions = UsageOptionsA

    def test_a(self):
        self.report['bar'] = 'bar'


class DummyTestCaseB(NetTestCase):

    usageOptions = UsageOptionsB

    def test_b(self):
        self.report['foo'] = 'foo'
"""

net_test_root_required = net_test_string + """
    requiresRoot = True
"""

net_test_string_with_file = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptions(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

class DummyTestCase(NetTestCase):
    inputFile = ['file', 'f', None, 'The input File']

    usageOptions = UsageOptions

    def test_a(self):
        self.report['bar'] = 'bar'

    def test_b(self):
        self.report['foo'] = 'foo'
"""

net_test_string_with_required_option = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptions(usage.Options):
    optParameters = [['spam', 's', None, 'ham'],
                     ['foo', 'o', None, 'moo'],
                     ['bar', 'o', None, 'baz'],
    ]

class DummyTestCase(NetTestCase):
    inputFile = ['file', 'f', None, 'The input File']

    requiredOptions = ['foo', 'bar']
    usageOptions = UsageOptions

    def test_a(self):
        self.report['bar'] = 'bar'

    def test_b(self):
        self.report['foo'] = 'foo'
"""

http_net_test = """
from twisted.internet import defer
from twisted.python import usage, failure

from ooni.utils import log
from ooni.utils.net import userAgents
from ooni.templates import httpt
from ooni.errors import failureToString, handleAllFailures

class UsageOptions(usage.Options):
    optParameters = [
                     ['url', 'u', None, 'Specify a single URL to test.'],
                    ]

class HTTPBasedTest(httpt.HTTPTest):
    usageOptions = UsageOptions
    def test_get(self):
        return self.doRequest(self.localOptions['url'], method="GET",
                              use_tor=False)
"""

generator_net_test = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptions(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

def input_generator():
    # Generates a list of numbers
    # The first value sent back is appended to the list.
    received = False
    numbers = [i for i in range(10)]
    while numbers:
        i = numbers.pop()
        result = yield i
        # Place sent value back in numbers
        if result is not None and received is False:
            numbers.append(result)
            received = True
            yield i

class TestSendGen(NetTestCase):
    usageOptions = UsageOptions
    inputs = input_generator()

    def test_input_sent_to_generator(self):
        # Sends a single value back to the generator
        if self.input == 5:
            self.inputs.send(self.input)
"""

generator_id_net_test = """
from twisted.python import usage
from ooni.nettest import NetTestCase

class UsageOptions(usage.Options):
    optParameters = [['spam', 's', None, 'ham']]

class DummyTestCaseA(NetTestCase):

    usageOptions = UsageOptions

    def test_a(self):
        self.report.setdefault("results", []).append(id(self.inputs))

    def test_b(self):
        self.report.setdefault("results", []).append(id(self.inputs))

    def test_c(self):
        self.report.setdefault("results", []).append(id(self.inputs))

class DummyTestCaseB(NetTestCase):

    usageOptions = UsageOptions

    def test_a(self):
        self.report.setdefault("results", []).append(id(self.inputs))
"""

dummyInputs = range(1)
dummyArgs = ('--spam', 'notham')
dummyOptions = {'spam': 'notham'}
dummyInvalidArgs = ('--cram', 'jam')
dummyInvalidOptions = {'cram': 'jam'}
dummyArgsWithRequiredOptions = ('--foo', 'moo', '--bar', 'baz')
dummyRequiredOptions = {'foo': 'moo', 'bar': 'baz'}
dummyArgsWithFile = ('--spam', 'notham', '--file', 'dummyInputFile.txt')
dummyInputFile = 'dummyInputFile.txt'



class TestNetTest(ConfigTestCase):
    timeout = 1

    def setUp(self):
        self.filename = ""
        with open(dummyInputFile, 'w') as f:
            for i in range(10):
                f.write("%s\n" % i)
        super(TestNetTest, self).setUp()

    def tearDown(self):
        os.remove(dummyInputFile)
        if self.filename != "":
            os.remove(self.filename)
        super(TestNetTest, self).tearDown()

    def assertCallable(self, thing):
        self.assertIn('__call__', dir(thing))

    def verifyMethods(self, testCases):
        uniq_test_methods = set()
        for test_class, test_methods in testCases:
            instance = test_class()
            for test_method in test_methods:
                c = getattr(instance, test_method)
                self.assertCallable(c)
                uniq_test_methods.add(test_method)
        self.assertEqual(set(['test_a', 'test_b']), uniq_test_methods)

    def test_load_net_test_from_file(self):
        """
        Given a file verify that the net test cases are properly
        generated.
        """
        __, net_test_file = mkstemp()
        with open(net_test_file, 'w') as f:
            f.write(net_test_string)
        f.close()

        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestFile(net_test_file)

        self.verifyMethods(ntl.getTestCases())
        os.unlink(net_test_file)

    def test_load_net_test_from_str(self):
        """
        Given a file like object verify that the net test cases are properly
        generated.
        """
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(net_test_string)

        self.verifyMethods(ntl.getTestCases())

    def test_load_net_test_multiple(self):
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(double_net_test_string)
        test_cases = ntl.getTestCases()
        self.verifyMethods(test_cases)
        ntl.checkOptions()

    def test_load_net_test_multiple_different_options(self):
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(double_different_options_net_test_string)

        test_cases = ntl.getTestCases()
        self.verifyMethods(test_cases)
        self.assertRaises(IncoherentOptions, ntl.checkOptions)

    def test_load_with_option(self):
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(net_test_string)

        self.assertIsInstance(ntl, NetTestLoader)
        for test_klass, test_meth in ntl.getTestCases():
            for option in dummyOptions.keys():
                self.assertIn(option, test_klass.usageOptions())

    def test_load_with_invalid_option(self):
        ntl = NetTestLoader(dummyInvalidArgs)
        ntl.loadNetTestString(net_test_string)
        self.assertRaises(UsageError, ntl.checkOptions)
        self.assertRaises(OONIUsageError, ntl.checkOptions)

    def test_load_with_required_option(self):
        ntl = NetTestLoader(dummyArgsWithRequiredOptions)
        ntl.loadNetTestString(net_test_string_with_required_option)

        self.assertIsInstance(ntl, NetTestLoader)

    def test_load_with_missing_required_option(self):
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(net_test_string_with_required_option)
        self.assertRaises(MissingRequiredOption, ntl.checkOptions)

    @defer.inlineCallbacks
    def test_net_test_inputs(self):
        ntl = NetTestLoader(dummyArgsWithFile)
        ntl.loadNetTestString(net_test_string_with_file)
        ntl.checkOptions()
        nt = NetTest(ntl.getTestCases(), ntl.getTestDetails(), None)
        yield nt.initialize()

        for test_class, test_methods in nt.testCases:
            self.assertEqual(len(list(test_class.inputs)), 10)

    def test_setup_local_options_in_test_cases(self):
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(net_test_string)

        ntl.checkOptions()
        self.assertEqual(dict(ntl.localOptions), dummyOptions)

    @defer.inlineCallbacks
    def test_generate_measurements_size(self):
        ntl = NetTestLoader(dummyArgsWithFile)
        ntl.loadNetTestString(net_test_string_with_file)
        ntl.checkOptions()

        net_test = NetTest(ntl.getTestCases(), ntl.getTestDetails(), None)

        yield net_test.initialize()
        measurements = list(net_test.generateMeasurements())
        self.assertEqual(len(measurements), 20)

    def test_net_test_completed_callback(self):
        ntl = NetTestLoader(dummyArgsWithFile)
        ntl.loadNetTestString(net_test_string_with_file)

        ntl.checkOptions()
        director = Director()

        self.filename = 'dummy_report.yamloo'
        d = director.startNetTest(ntl, self.filename)

        @d.addCallback
        def complete(result):
            self.assertEqual(result, None)
            self.assertEqual(director.successfulMeasurements, 20)

        return d

    def test_require_root_succeed(self):
        # XXX: will require root to run
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(net_test_root_required)

        for test_class, methods in ntl.getTestCases():
            self.assertTrue(test_class.requiresRoot)

    def test_singular_input_processor(self):
        """
        Verify that all measurements use the same object as their input processor.
        """
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(generator_id_net_test)
        ntl.checkOptions()

        director = Director()
        self.filename = 'dummy_report.yamloo'
        d = director.startNetTest(ntl, self.filename)

        @d.addCallback
        def complete(result):
            with open(self.filename) as report_file:
                all_report_entries = yaml.safe_load_all(report_file)
                header = all_report_entries.next()
                results_case_a = all_report_entries.next()
                aa_test, ab_test, ac_test = results_case_a.get('results', [])
                results_case_b = all_report_entries.next()
                ba_test = results_case_b.get('results', [])[0]
            # Within a NetTestCase an inputs object will be consistent
            self.assertEqual(aa_test, ab_test, ac_test)
            # An inputs object will be different between different NetTestCases
            self.assertNotEqual(aa_test, ba_test)

        return d

    def test_send_to_inputs_generator(self):
        """
        Verify that a net test can send information back into an inputs generator.
        """
        ntl = NetTestLoader(dummyArgs)
        ntl.loadNetTestString(generator_net_test)
        ntl.checkOptions()

        director = Director()
        self.filename = 'dummy_report.yamloo'
        d = director.startNetTest(ntl, self.filename)

        @d.addCallback
        def complete(result):
            with open(self.filename) as report_file:
                all_report_entries = yaml.safe_load_all(report_file)
                header = all_report_entries.next()
                results = [x['input'] for x in all_report_entries]
            self.assertEqual(results, [9, 8, 7, 6, 5, 5, 3, 2, 1, 0])

        return d

class TestNettestTimeout(ConfigTestCase):

    @defer.inlineCallbacks
    def setUp(self):
        super(TestNettestTimeout, self).setUp()
        from twisted.internet.protocol import Protocol, Factory
        from twisted.internet.endpoints import TCP4ServerEndpoint

        class DummyProtocol(Protocol):

            def dataReceived(self, data):
                pass

        class DummyFactory(Factory):

            def __init__(self):
                self.protocols = []

            def buildProtocol(self, addr):
                proto = DummyProtocol()
                self.protocols.append(proto)
                return proto

            def stopFactory(self):
                for proto in self.protocols:
                    proto.transport.loseConnection()

        self.factory = DummyFactory()
        endpoint = TCP4ServerEndpoint(reactor, 8007)
        self.port = yield endpoint.listen(self.factory)

        config.advanced.measurement_timeout = 2

    def tearDown(self):
        super(TestNettestTimeout, self).tearDown()
        self.factory.stopFactory()
        self.port.stopListening()
        os.remove(self.filename)

    def test_nettest_timeout(self):
        ntl = NetTestLoader(('-u', 'http://localhost:8007/'))
        ntl.loadNetTestString(http_net_test)

        ntl.checkOptions()
        director = Director()

        self.filename = 'dummy_report.yamloo'
        d = director.startNetTest(ntl, self.filename)

        @d.addCallback
        def complete(result):
            assert director.failedMeasurements == 1

        return d



# -*- encoding: utf-8 -*-
from twisted.internet.protocol import Factory, Protocol
from twisted.internet.endpoints import TCP4ClientEndpoint

from ooni import nettest
from ooni.errors import handleAllFailures
from ooni.utils import log


class TCPFactory(Factory):

    def buildProtocol(self, addr):
        return Protocol()


class TCPConnectTest(nettest.NetTestCase):
    name = "TCP Connect"
    description = "Performs a TCP connect scan of all the " \
                  "host port combinations given as input."
    author = "Arturo Filastò"
    version = "0.1"
    inputFile = [
        'file',
        'f',
        None,
        'File containing the IP:PORT combinations to be tested, one per line.']

    requiresTor = False
    requiresRoot = False
    requiredOptions = ['file']

    def test_connect(self):
        """
        This test performs a TCP connection to the remote host on the
        specified port.
        The report will contains the string 'success' if the test has
        succeeded, or the reason for the failure if it has failed.
        """
        host, port = self.input.split(":")

        def connectionSuccess(protocol):
            protocol.transport.loseConnection()
            log.debug("Got a connection to %s" % self.input)
            self.report["connection"] = 'success'

        def connectionFailed(failure):
            self.report['connection'] = handleAllFailures(failure)

        from twisted.internet import reactor
        point = TCP4ClientEndpoint(reactor, host, int(port))
        d = point.connect(TCPFactory())
        d.addCallback(connectionSuccess)
        d.addErrback(connectionFailed)
        return d

    def inputProcessor(self, filename=None):
        """
        This inputProcessor extracts name:port pairs from urls
        XXX: Does not support unusual port numbers
        """
        def strip_url(address):
            proto, path = x.strip().split('://')
            proto = proto.lower()
            host = path.split('/')[0]
            if proto == 'http':
                return "%s:80" % host
            if proto == 'https':
                return "%s:443" % host

        pluggable_transports = ("obfs3", "obfs2", "fte", "scramblesuit")
        def is_bridge_line(line):
            first = line.split(" ")[0]
            return first.lower() in pluggable_transports + ("bridge",)
        def strip_bridge(line):
            if line.lower().startswith("Bridge"):
                return line.split(" ")[2]
            return line.split(" ")[1]

        if filename:
            fp = open(filename)
            for x in fp.readlines():
                if x.startswith("http"):
                    yield strip_url(x)
                elif is_bridge_line(x):
                    yield strip_bridge(x)
                else:
                    yield x.split(" ")[0]
            fp.close()
        else:
            pass



from twisted.internet.protocol import Factory, Protocol

class TCPConnectProtocol(Protocol):
    def connectionMade(self):
        self.transport.loseConnection()

class TCPConnectFactory(Factory):
    noisy = False
    def buildProtocol(self, addr):
        return TCPConnectProtocol()



import os
import csv
from ooni.settings import config


def load_input(file_input, file_output):
    fw = open(file_output, "w+")
    with open(file_input) as f:
        csvreader = csv.reader(f)
        csvreader.next()
        for row in csvreader:
            fw.write("%s\n" % row[0])
    fw.close()


def generate_country_input(country_code, dst):
    """
    Write to dst/citizenlab-urls-{country_code}.txt
    the list for the given country code.

    Returns:

        the path to the generated input
    """

    country_code = country_code.lower()
    filename = os.path.join(dst, "citizenlab-urls-%s.txt" % country_code)

    input_list = config.get_data_file_path("resources/"
                                           "citizenlab-test-lists/"
                                           + country_code + ".csv")

    if not input_list:
        raise Exception("Could not find list for country %s" % country_code)

    load_input(input_list, filename)

    return filename


def generate_global_input(dst):
    filename = os.path.join(dst, "citizenlab-urls-global.txt")

    input_list = config.get_data_file_path("resources/"
                                           "citizenlab-test-lists/"
                                           "global.csv")

    if not input_list:
        print("Could not find the global input list")
        print("Perhaps you should run ooniresources")
        raise Exception("Could not find the global input list")

    load_input(input_list, filename)

    return filename



from twisted.internet import protocol, defer, reactor
from twisted.internet.endpoints import TCP4ClientEndpoint

from ooni.nettest import NetTestCase
from ooni.errors import failureToString
from ooni.utils import log

class TCPSender(protocol.Protocol):
    def __init__(self):
        self.received_data = ''
        self.sent_data = ''

    def dataReceived(self, data):
        """
        We receive data until the total amount of data received reaches that
        which we have sent. At that point we append the received data to the
        report and we fire the callback of the test template sendPayload
        function.

        This is used in pair with a TCP Echo server.

        The reason why we put the data received inside of an array is that in
        future we may want to expand this to support state and do something
        similar to what daphne does, but without the mutation.

        XXX Actually daphne will probably be refactored to be a subclass of the
        TCP Test Template.
        """
        if self.payload_len:
            self.received_data += data

    def sendPayload(self, payload):
        """
        Write the payload to the wire and set the expected size of the payload
        we are to receive.

        Args:

            payload: the data to be sent on the wire.

        """
        self.payload_len = len(payload)
        self.sent_data = payload
        self.transport.write(payload)

class TCPSenderFactory(protocol.Factory):
    noisy = False
    def buildProtocol(self, addr):
        return TCPSender()

class TCPTest(NetTestCase):
    name = "Base TCP Test"
    version = "0.1"

    requiresRoot = False
    timeout = 5
    address = None
    port = None

    def _setUp(self):
        super(TCPTest, self)._setUp()

        self.report['sent'] = []
        self.report['received'] = []

    def sendPayload(self, payload):
        d1 = defer.Deferred()

        def closeConnection(proto):
            self.report['sent'].append(proto.sent_data)
            self.report['received'].append(proto.received_data)
            proto.transport.loseConnection()
            log.debug("Closing connection")
            d1.callback(proto.received_data)

        def timedOut(proto):
            self.report['failure'] = 'tcp_timed_out_error'
            proto.transport.loseConnection()

        def errback(failure):
            self.report['failure'] = failureToString(failure)
            d1.errback(failure)

        def connected(proto):
            log.debug("Connected to %s:%s" % (self.address, self.port))
            proto.report = self.report
            proto.deferred = d1
            proto.sendPayload(payload)
            if self.timeout:
                # XXX-Twisted this logic should probably go inside of the protocol
                reactor.callLater(self.timeout, closeConnection, proto)

        point = TCP4ClientEndpoint(reactor, self.address, self.port)
        log.debug("Connecting to %s:%s" % (self.address, self.port))
        d2 = point.connect(TCPSenderFactory())
        d2.addCallback(connected)
        d2.addErrback(errback)
        return d1




# -*- encoding: utf-8 -*-
#
# :licence: see LICENSE

from twisted.python import usage
from ooni.templates import httpt
from ooni.utils import log

class UsageOptions(usage.Options):
    optParameters = [ ['expectedBody', 'B',
                         'I’m just a happy little web server.\n',
                          'Expected body content from GET response.'],
                      ['domainName', 'D', None,
                        'Specify a single fronted domainName to test.'],
                      ['hostHeader', 'H', None,
                        'Specify "inside" Host Header to test.']
                    ]

class meekTest(httpt.HTTPTest):
    """
    Performs a HTTP GET request to a list of fronted domains with the Host
    Header of the "inside" meek-server. The meek-server handles a GET request
    and response with: "I’m just a happy little web server.\n".
    The input file should be formatted as (one per line):
    "domainName:hostHeader"
    ajax.aspnetcdn.com:az668014.vo.msecnd.net
    a0.awsstatic.com:d2zfqthxsdq309.cloudfront.net

    """
    name = "Meek fronted requests test"
    description = "This tests for the Meek Tor pluggable transport "\
                  "frontend reachability."
    version = "0.0.1"

    usageOptions = UsageOptions
    inputFile = ['file', 'f', None,
                  "File containing the domainName:hostHeader combinations to\
                  be tested, one per line."]
    inputs = [('ajax.aspnetcdn.com', 'az668014.vo.msecnd.net'),
               ('a0.awsstatic.com', 'd2zfqthxsdq309.cloudfront.net')]

    requiresRoot = False
    requiresTor = False

    def setUp(self):
        """
        Check for inputs.
        """

        if self.input:
           if (isinstance(self.input, tuple) or isinstance(self.input, list)):
               self.domainName, self.header = self.input
           else:
               self.domainName, self.header = self.input.split(':')
        elif (self.localOptions['domainName'] and
              self.localOptions['hostHeader']):
               self.domainName = self.localOptions['domainName']
               self.header = self.localOptions['hostHeader']

        self.expectedBody = self.localOptions['expectedBody']
        self.domainName = 'https://' + self.domainName

    def test_meek_response(self):
        """
        Detects if the fronted request is blocked.
        """
        log.msg("Testing fronted domain:%s with Host Header:%s"
                % (self.domainName, self.header))
        def process_body(body):
            if self.expectedBody != body:
                self.report['success'] = False
            else:
                self.report['success'] = True

        headers = {}
        headers['Host'] = [self.header]
        return self.doRequest(self.domainName, method="GET", headers=headers,
                              body_processor=process_body)




import yaml

from twisted.trial import unittest

from ooni.reporter import OSafeDumper

from scapy.all import IP, UDP


class TestScapyRepresent(unittest.TestCase):
    def test_represent_scapy(self):
        data = IP() / UDP()
        yaml.dump_all([data], Dumper=OSafeDumper)





import os
import json

from urlparse import urljoin, urlparse

from twisted.web.error import Error
from twisted.web.client import Agent, Headers
from twisted.internet import defer, reactor
from twisted.internet.endpoints import TCP4ClientEndpoint

from twisted.python.versions import Version
from twisted import version as _twisted_version
_twisted_14_0_2_version = Version('twisted', 14, 0, 2)

from ooni import errors as e
from ooni.settings import config
from ooni.utils import log, onion
from ooni.utils.net import BodyReceiver, StringProducer, Downloader
from ooni.utils.socks import TrueHeadersSOCKS5Agent


def guess_backend_type(address):
    if address is None:
        raise e.InvalidAddress
    if onion.is_onion_address(address):
        return 'onion'
    elif address.startswith('https://'):
        return 'https'
    elif address.startswith('http://'):
        return 'http'
    else:
        raise e.InvalidAddress

class OONIBClient(object):
    def __init__(self, address=None, settings={}):
        self.base_headers = {}
        self.backend_type = settings.get('type', None)
        self.base_address = settings.get('address', address)
        self.front = settings.get('front', '').encode('ascii')

        if self.backend_type is None:
            self.backend_type = guess_backend_type(self.base_address)
        self.backend_type = self.backend_type.encode('ascii')

        self.settings = {
            'type': self.backend_type,
            'address': self.base_address,
            'front': self.front
        }
        self._setupBaseAddress()

    def _setupBaseAddress(self):
        parsed_address = urlparse(self.base_address)
        if self.backend_type == 'onion':
            if not onion.is_onion_address(self.base_address):
                log.err("Invalid onion address.")
                raise e.InvalidAddress(self.base_address)
            if parsed_address.scheme in ('http', 'httpo'):
                self.base_address = ("http://%s" % parsed_address.netloc)
            else:
                self.base_address = ("%s://%s" % (parsed_address.scheme,
                                                  parsed_address.netloc))
        elif self.backend_type == 'http':
            self.base_address = ("http://%s" % parsed_address.netloc)
        elif self.backend_type == 'https':
            self.base_address = ("https://%s" % parsed_address.netloc)
        elif self.backend_type == 'cloudfront':
            self.base_headers['Host'] = [parsed_address.netloc]
            self.base_address = ("https://%s" % self.front)
        self.base_address = self.base_address.encode('ascii')

    def isSupported(self):
        if self.backend_type in ("https", "cloudfront"):
            if _twisted_version < _twisted_14_0_2_version:
                log.err("HTTPS and cloudfronted backends require "
                        "twisted > 14.0.2.")
                return False
        elif self.backend_type == "http":
            if config.advanced.insecure_backend is not True:
                log.err("Plaintext backends are not supported. To "
                        "enable at your own risk set "
                        "advanced->insecure_backend to true")
                return False
        elif self.backend_type == "onion":
            # XXX add an extra check to ensure tor is running
            if not config.tor_state and config.tor.socks_port is None:
                return False
        return True

    def isReachable(self):
        raise NotImplemented

    def _request(self, method, urn, genReceiver, bodyProducer=None, retries=3):
        if self.backend_type == 'onion':
            agent = TrueHeadersSOCKS5Agent(reactor,
                                           proxyEndpoint=TCP4ClientEndpoint(reactor,
                                                                            '127.0.0.1',
                                                                            config.tor.socks_port))
        else:
            agent = Agent(reactor)

        attempts = 0

        finished = defer.Deferred()

        def perform_request(attempts):
            uri = urljoin(self.base_address, urn)
            d = agent.request(method, uri, bodyProducer=bodyProducer,
                              headers=Headers(self.base_headers))

            @d.addCallback
            def callback(response):
                try:
                    content_length = int(response.headers.getRawHeaders('content-length')[0])
                except:
                    content_length = None
                response.deliverBody(genReceiver(finished, content_length))

            def errback(err, attempts):
                # We we will recursively keep trying to perform a request until
                # we have reached the retry count.
                if attempts < retries:
                    log.err("Lookup failed. Retrying.")
                    attempts += 1
                    perform_request(attempts)
                else:
                    log.err("Failed. Giving up.")
                    finished.errback(err)

            d.addErrback(errback, attempts)

        perform_request(attempts)

        return finished

    def queryBackend(self, method, urn, query=None, retries=3):
        bodyProducer = None
        if query:
            bodyProducer = StringProducer(json.dumps(query))

        def genReceiver(finished, content_length):
            def process_response(s):
                # If empty string then don't parse it.
                if not s:
                    return
                try:
                    response = json.loads(s)
                except ValueError:
                    raise e.get_error(None)
                if 'error' in response:
                    log.debug("Got this backend error message %s" % response)
                    raise e.get_error(response['error'])
                return response

            return BodyReceiver(finished, content_length, process_response)

        return self._request(method, urn, genReceiver, bodyProducer, retries)

    def download(self, urn, download_path):

        def genReceiver(finished, content_length):
            return Downloader(download_path, finished, content_length)

        return self._request('GET', urn, genReceiver)

class BouncerClient(OONIBClient):
    def isReachable(self):
        return defer.succeed(True)

    @defer.inlineCallbacks
    def lookupTestCollector(self, net_tests):
        try:
            test_collector = yield self.queryBackend('POST', '/bouncer/net-tests',
                                                     query={'net-tests': net_tests})
        except Exception as exc:
            log.exception(exc)
            raise e.CouldNotFindTestCollector

        defer.returnValue(test_collector)

    @defer.inlineCallbacks
    def lookupTestHelpers(self, test_helper_names):
        try:
            test_helper = yield self.queryBackend('POST', '/bouncer/test-helpers',
                                                  query={'test-helpers': test_helper_names})
        except Exception as exc:
            log.exception(exc)
            raise e.CouldNotFindTestHelper

        if not test_helper:
            raise e.CouldNotFindTestHelper

        defer.returnValue(test_helper)


class CollectorClient(OONIBClient):
    def isReachable(self):
        # XXX maybe in the future we can have a dedicated API endpoint to
        # test the reachability of the collector.
        d = self.queryBackend('GET', '/invalidpath')

        @d.addCallback
        def cb(_):
            # We should never be getting an acceptable response for a
            # request to an invalid path.
            return False

        @d.addErrback
        def err(failure):
            failure.trap(Error)
            return failure.value.status == '404'

        return d

    def getInput(self, input_hash):
        from ooni.deck import InputFile

        input_file = InputFile(input_hash)
        if input_file.descriptorCached:
            return defer.succeed(input_file)
        else:
            d = self.queryBackend('GET', '/input/' + input_hash)

            @d.addCallback
            def cb(descriptor):
                input_file.load(descriptor)
                input_file.save()
                return input_file

            @d.addErrback
            def err(err):
                log.err("Failed to get descriptor for input %s" % input_hash)
                log.exception(err)

            return d

    def getInputList(self):
        return self.queryBackend('GET', '/input')

    def downloadInput(self, input_hash):
        from ooni.deck import InputFile

        input_file = InputFile(input_hash)

        if input_file.fileCached:
            return defer.succeed(input_file)
        else:
            d = self.download('/input/' + input_hash + '/file', input_file.cached_file)

            @d.addCallback
            def cb(res):
                input_file.verify()
                return input_file

            @d.addErrback
            def err(err):
                log.err("Failed to download the input file %s" % input_hash)
                log.exception(err)

            return d

    def getInputPolicy(self):
        return self.queryBackend('GET', '/policy/input')

    def getNettestPolicy(self):
        return self.queryBackend('GET', '/policy/nettest')

    def getDeckList(self):
        return self.queryBackend('GET', '/deck')

    def getDeck(self, deck_hash):
        from ooni.deck import Deck

        deck = Deck(deck_hash)
        if deck.descriptorCached:
            return defer.succeed(deck)
        else:
            d = self.queryBackend('GET', '/deck/' + deck_hash)

            @d.addCallback
            def cb(descriptor):
                deck.load(descriptor)
                deck.save()
                return deck

            @d.addErrback
            def err(err):
                log.err("Failed to get descriptor for deck %s" % deck_hash)
                log.exception(err)

            return d

    def downloadDeck(self, deck_hash):
        from ooni.deck import Deck

        deck = Deck(deck_hash)
        if deck.fileCached:
            return defer.succeed(deck)
        else:
            d = self.download('/deck/' + deck_hash + '/file', deck.cached_file)

            @d.addCallback
            def cb(res):
                deck.verify()
                return deck

            @d.addErrback
            def err(err):
                log.err("Failed to download the deck %s" % deck_hash)
                log.exception(err)

            return d

    def createReport(self, test_details):
        request = {
            'software_name': test_details['software_name'],
            'software_version': test_details['software_version'],
            'probe_asn': test_details['probe_asn'],
            'probe_cc': test_details['probe_cc'],
            'test_name': test_details['test_name'],
            'test_version': test_details['test_version'],
            'test_start_time': test_details['test_start_time'],
            'input_hashes': test_details['input_hashes'],
            'data_format_version': test_details['data_format_version'],
            'format': 'json'
        }
        # import values from the environment
        request.update([(k.lower(),v) for (k,v) in os.environ.iteritems()
                        if k.startswith('PROBE_')])

        return self.queryBackend('POST', '/report', query=request)

    def updateReport(self, report_id, serialization_format, entry_content):
        request = {
            'format': serialization_format,
            'content': entry_content
        }
        return self.queryBackend('POST', '/report/%s' % report_id,
                                 query=request)


    def closeReport(self, report_id):
        return self.queryBackend('POST', '/report/' + report_id + '/close')

class WebConnectivityClient(OONIBClient):
    def isReachable(self):
        d = self.queryBackend('GET', '/status')

        @d.addCallback
        def cb(result):
            if result.get("status", None) != "ok":
                return False
            return True

        @d.addErrback
        def err(_):
            return False

        return d

    def control(self, http_request, tcp_connect):
        request = {
            'http_request': http_request,
            'tcp_connect': tcp_connect
        }
        return self.queryBackend('POST', '/', query=request)



import sys

from twisted.internet import defer
from twisted.python import usage

from ooni.resources import __version__
from ooni.resources import update


class Options(usage.Options):
    synopsis = """%s
    This is used to update the resources required to run oonideckgen and
    ooniprobe.
    You just run this script with no arguments and it will update the
    resources.
    """ % sys.argv[0]

    optFlags = [
        ["update-inputs", None, "(deprecated) update the resources needed for "
                                "inputs."],
        ["update-geoip", None, "(deprecated) Update the geoip related "
                               "resources."]
    ]
    optParameters = []

    def opt_version(self):
        print("ooniresources version: %s" % __version__)
        sys.exit(0)


@defer.inlineCallbacks
def run():
    options = Options()
    try:
        options.parseOptions()
    except usage.UsageError as error_message:
        print "%s: %s" % (sys.argv[0], error_message)
        print "%s: Try --help for usage details." % (sys.argv[0])
        sys.exit(1)

    if options['update-inputs'] or options['update-geoip']:
        print("WARNING: Passing command line arguments is deprecated")

    yield update.download_resources()



import re
import codecs
from base64 import b64encode

META_CHARSET_REGEXP = re.compile('<meta(?!\s*(?:name|value)\s*=)[^>]*?charset\s*=[\s"\']*([^\s"\'/>!;]+)')

def representBody(body):
    if not body:
        return body
    # XXX perhaps add support for decoding gzip in the future.
    body = body.replace('\0', '')
    decoded = False
    charsets = ['ascii', 'utf-8']

    # If we are able to detect the charset of body from the meta tag
    # try to decode using that one first
    charset = META_CHARSET_REGEXP.search(body, re.IGNORECASE)
    if charset:
        try:
            encoding = charset.group(1).lower()
            codecs.lookup(encoding)
            charsets.insert(0, encoding)
        except (LookupError, IndexError):
            # Skip invalid codecs and partial regexp match
            pass

    for encoding in charsets:
        try:
            body = unicode(body, encoding)
            decoded = True
            break
        except UnicodeDecodeError:
            pass
    if not decoded:
        body = {
            'data': b64encode(body),
            'format': 'base64'
        }
    return body

TITLE_REGEXP = re.compile("<title>(.*?)</title>", re.IGNORECASE | re.DOTALL)

def extractTitle(body):
    m = TITLE_REGEXP.search(body, re.IGNORECASE | re.DOTALL)
    if m:
        return unicode(m.group(1), errors='ignore')
    return ''

REQUEST_HEADERS = {
    'User-Agent': ['Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, '
                   'like Gecko) Chrome/47.0.2526.106 Safari/537.36'],
    'Accept-Language': ['en-US;q=0.8,en;q=0.5'],
    'Accept': ['text/html,application/xhtml+xml,application/xml;q=0.9,'
               '*/*;q=0.8']
}



# -*- encoding: utf-8 -*-
from twisted.python import usage

from ooni.utils import log
from ooni.utils import randomStr, randomSTR
from ooni.templates import tcpt


class UsageOptions(usage.Options):
    optParameters = [
        ['backend', 'b', None, 'The OONI backend that runs a TCP echo server.'],
        ['backendport', 'p', 80,
         'Specify the port that the TCP echo server is running '
         '(should only be set for debugging).']]


class HTTPInvalidRequestLine(tcpt.TCPTest):

    """
    The goal of this test is to do some very basic and not very noisy fuzzing
    on the HTTP request line. We generate a series of requests that are not
    valid HTTP requests.

    Unless elsewhere stated 'Xx'*N refers to N*2 random upper or lowercase
    ascii letters or numbers ('XxXx' will be 4).
    """
    name = "HTTP Invalid Request Line"
    description = "Performs out of spec HTTP requests in the attempt to "\
                  "trigger a proxy error message."
    version = "0.2"
    authors = "Arturo Filastò"

    usageOptions = UsageOptions

    requiredTestHelpers = {'backend': 'tcp-echo'}
    requiredOptions = ['backend']
    requiresRoot = False
    requiresTor = False

    def setUp(self):
        self.port = int(self.localOptions['backendport'])
        self.address = self.localOptions['backend']
        self.report['tampering'] = None

    def check_for_manipulation(self, response, payload):
        log.debug("Checking if %s == %s" % (response, payload))
        if response != payload:
            log.msg("Detected manipulation!")
            log.msg(response)
            self.report['tampering'] = True
        else:
            log.msg("No manipulation detected.")
            self.report['tampering'] = False

    def test_random_invalid_method(self):
        """
        We test sending data to a TCP echo server listening on port 80, if what
        we get back is not what we have sent then there is tampering going on.
        This is for example what squid will return when performing such
        request:

            HTTP/1.0 400 Bad Request
            Server: squid/2.6.STABLE21
            Date: Sat, 23 Jul 2011 02:22:44 GMT
            Content-Type: text/html
            Content-Length: 1178
            Expires: Sat, 23 Jul 2011 02:22:44 GMT
            X-Squid-Error: ERR_INVALID_REQ 0
            X-Cache: MISS from cache_server
            X-Cache-Lookup: NONE from cache_server:3128
            Via: 1.0 cache_server:3128 (squid/2.6.STABLE21)
            Proxy-Connection: close

        """
        payload = randomSTR(4) + " / HTTP/1.1\n\r"

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_random_invalid_field_count(self):
        """
        This generates a request that looks like this:

        XxXxX XxXxX XxXxX XxXxX

        This may trigger some bugs in the HTTP parsers of transparent HTTP
        proxies.
        """
        payload = ' '.join(randomStr(5) for x in range(4))
        payload += "\n\r"

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_random_big_request_method(self):
        """
        This generates a request that looks like this:

        Xx*512 / HTTP/1.1
        """
        payload = randomStr(1024) + ' / HTTP/1.1\n\r'

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_random_invalid_version_number(self):
        """
        This generates a request that looks like this:

        GET / HTTP/XxX
        """
        payload = 'GET / HTTP/' + randomStr(3)
        payload += '\n\r'

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d



# -*- encoding: utf-8 -*-
from twisted.python import usage
from twisted.internet import defer

from ooni.templates import dnst
from ooni.utils import log

class UsageOptions(usage.Options):
    optParameters = [
            ['resolver', 'r', '8.8.8.1', 'an invalid DNS resolver'],
            ['timeout', 't', 3, 'timeout after which we should consider the query failed']
    ]

class DNSInjectionTest(dnst.DNSTest):
    """
    This test detects DNS spoofed DNS responses by performing UDP based DNS
    queries towards an invalid DNS resolver.

    For it to work we must be traversing the network segment of a machine that
    is actively injecting DNS query answers.
    """
    name = "DNS Injection"
    description = "Checks for injection of spoofed DNS answers"
    version = "0.1"
    authors = "Arturo Filastò"

    inputFile = ['file', 'f', None,
                 'Input file of list of hostnames to attempt to resolve']

    usageOptions = UsageOptions
    requiredOptions = ['resolver', 'file']
    requiresRoot = False
    requiresTor = False

    def setUp(self):
        self.resolver = (self.localOptions['resolver'], 53)
        self.queryTimeout = [self.localOptions['timeout']]

    def inputProcessor(self, filename):
        fp = open(filename)
        for line in fp:
            if line.startswith('http://'):
                yield line.replace('http://', '').replace('/', '').strip()
            else:
                yield line.strip()
        fp.close()

    def test_injection(self):
        self.report['injected'] = None

        d = self.performALookup(self.input, self.resolver)
        @d.addCallback
        def cb(res):
            log.msg("The DNS query for %s is injected" % self.input)
            self.report['injected'] = True

        @d.addErrback
        def err(err):
            err.trap(defer.TimeoutError)
            log.msg("The DNS query for %s is not injected" % self.input)
            self.report['injected'] = False

        return d




import yaml

from mock import patch, MagicMock

from twisted.internet import defer
from ooni.tests.bases import ConfigTestCase

from ooni.report import tool

mock_tor_check = MagicMock(return_value=True)

class TestOONIReport(ConfigTestCase):

    def _create_reporting_yaml(self, filename):
        from ooni.settings import config
        with open(config.report_log_file, 'w+') as f:
            yaml.dump({
                filename: {
                    "collector": "httpo://thirteenchars123.onion"
                }
            }, f)

    def _write_dummy_report(self, filename):
        from ooni.reporter import YAMLReporter
        from .test_reporter import test_details
        reporter = YAMLReporter(test_details, filename)
        reporter.createReport()
        reporter.writeReportEntry({"spam": "ham"})
        reporter.finish()

    def test_cli_status(self):
        mock_tool = MagicMock()
        with patch('ooni.report.cli.tool', mock_tool):
            from ooni.report import cli
            cli.run(["status"])
            self.assertTrue(mock_tool.status.called)

    @patch('ooni.report.cli.tor_check', mock_tor_check)
    def test_cli_upload(self):
        mock_tool = MagicMock()
        with patch('ooni.report.cli.tool', mock_tool):
            from ooni.report import cli
            cli.run(["upload", "dummy.yaml"])
            self.assertTrue(mock_tool.upload.called)

    @patch('ooni.report.cli.tor_check', mock_tor_check)
    def test_cli_upload_all(self):
        mock_tool = MagicMock()
        with patch('ooni.report.cli.tool', mock_tool):
            from ooni.report import cli
            cli.run(["upload"])
            self.assertTrue(mock_tool.upload_all.called)

    @patch('ooni.report.tool.CollectorClient')
    @patch('ooni.report.tool.OONIBReportLog')
    @patch('ooni.report.tool.OONIBReporter')
    def test_tool_upload(self, mock_oonib_reporter, mock_oonib_report_log,
                         mock_collector_client):

        mock_oonib_reporter_i = mock_oonib_reporter.return_value
        mock_oonib_reporter_i.createReport.return_value = defer.succeed("fake_id")
        mock_oonib_reporter_i.writeReportEntry.return_value = defer.succeed(True)
        mock_oonib_reporter_i.finish.return_value = defer.succeed(True)

        mock_oonib_report_log_i = mock_oonib_report_log.return_value
        mock_oonib_report_log_i.created.return_value = defer.succeed(True)
        mock_oonib_report_log_i.closed.return_value = defer.succeed(True)

        report_name = "dummy_report.yaml"
        self._create_reporting_yaml(report_name)
        self._write_dummy_report(report_name)

        d = tool.upload(report_name)
        @d.addCallback
        def cb(result):
            mock_oonib_reporter_i.writeReportEntry.assert_called_with(
                {"spam": "ham"}
            )
        return d

    @patch('ooni.report.tool.CollectorClient')
    @patch('ooni.report.tool.OONIBReportLog')
    @patch('ooni.report.tool.OONIBReporter')
    def test_tool_upload_all(self, mock_oonib_reporter, mock_oonib_report_log,
                         mock_collector_client):

        mock_oonib_reporter_i = mock_oonib_reporter.return_value
        mock_oonib_reporter_i.createReport.return_value = defer.succeed("fake_id")
        mock_oonib_reporter_i.writeReportEntry.return_value = defer.succeed(True)
        mock_oonib_reporter_i.finish.return_value = defer.succeed(True)

        mock_oonib_report_log_i = mock_oonib_report_log.return_value
        mock_oonib_report_log_i.created.return_value = defer.succeed(True)
        mock_oonib_report_log_i.closed.return_value = defer.succeed(True)
        mock_oonib_report_log_i.reports_to_upload = [("dummy_report.yaml", None)]

        report_name = "dummy_report.yaml"
        self._create_reporting_yaml(report_name)
        self._write_dummy_report(report_name)

        d = tool.upload_all()
        @d.addCallback
        def cb(result):
            mock_oonib_reporter_i.writeReportEntry.assert_called_with(
                {"spam": "ham"}
            )
        return d



# -*- encoding: utf-8 -*-
#
# :authors: Arturo Filastò
# :licence: see LICENSE

from twisted.internet import udp, error, base
from twisted.internet.defer import TimeoutError
from twisted.names import client, dns
from twisted.names.client import Resolver

from ooni.utils import log
from ooni.nettest import NetTestCase
from ooni.errors import failureToString

import socket
from socket import gaierror

dns.DNSDatagramProtocol.noisy = False

def _bindSocket(self):
    """
    _bindSocket taken from Twisted 13.1.0 to suppress logging.
    """
    try:
        skt = self.createInternetSocket()
        skt.bind((self.interface, self.port))
    except socket.error as le:
        raise error.CannotListenError(self.interface, self.port, le)

    # Make sure that if we listened on port 0, we update that to
    # reflect what the OS actually assigned us.
    self._realPortNumber = skt.getsockname()[1]

    # Here we remove the logging.
    # log.msg("%s starting on %s" % (
    #         self._getLogPrefix(self.protocol), self._realPortNumber))

    self.connected = 1
    self.socket = skt
    self.fileno = self.socket.fileno
udp.Port._bindSocket = _bindSocket

def connectionLost(self, reason=None):
    """
    Taken from Twisted 13.1.0 to suppress log.msg printing.
    """
    # Here we remove the logging.
    # log.msg('(UDP Port %s Closed)' % self._realPortNumber)
    self._realPortNumber = None
    base.BasePort.connectionLost(self, reason)
    self.protocol.doStop()
    self.socket.close()
    del self.socket
    del self.fileno
    if hasattr(self, "d"):
        self.d.callback(None)
        del self.d
udp.Port.connectionLost = connectionLost

def representAnswer(answer):
    answer_types = {
        dns.SOA: 'SOA',
        dns.NS: 'NS',
        dns.PTR: 'PTR',
        dns.A: 'A',
        dns.CNAME: 'CNAME',
        dns.MX: 'MX'
    }
    answer_type = answer_types.get(answer.type, 'unknown')
    represented_answer = {
        "answer_type": answer_type
    }
    if answer_type is 'SOA':
        represented_answer['ttl'] = answer.payload.ttl
        represented_answer['hostname'] = answer.payload.mname.name
        represented_answer['responsible_name'] = answer.payload.rname.name
        represented_answer['serial_number'] = answer.payload.serial
        represented_answer['refresh_interval'] = answer.payload.refresh
        represented_answer['retry_interval'] = answer.payload.retry
        represented_answer['minimum_ttl'] = answer.payload.minimum
        represented_answer['expiration_limit'] = answer.payload.expire
    elif answer_type in ['NS', 'PTR', 'CNAME']:
        represented_answer['hostname'] = answer.payload.name.name
    elif answer_type is 'A':
        represented_answer['ipv4'] = answer.payload.dottedQuad()
    return represented_answer

class DNSTest(NetTestCase):
    name = "Base DNS Test"
    version = "0.2.0"

    requiresRoot = False
    queryTimeout = [1]

    def _setUp(self):
        super(DNSTest, self)._setUp()

        self.report['queries'] = []

    def performPTRLookup(self, address, dns_server = None):
        """
        Does a reverse DNS lookup on the input ip address

        :address: the IP Address as a dotted quad to do a reverse lookup on.

        :dns_server: is the dns_server that should be used for the lookup as a
                     tuple of ip port (ex. ("127.0.0.1", 53))

                     if None, system dns settings will be used
        """
        ptr = '.'.join(address.split('.')[::-1]) + '.in-addr.arpa'
        return self.dnsLookup(ptr, 'PTR', dns_server)

    def performALookup(self, hostname, dns_server = None):
        """
        Performs an A lookup and returns an array containg all the dotted quad
        IP addresses in the response.

        :hostname: is the hostname to perform the A lookup on

        :dns_server: is the dns_server that should be used for the lookup as a
                     tuple of ip port (ex. ("127.0.0.1", 53))

                     if None, system dns settings will be used
        """
        return self.dnsLookup(hostname, 'A', dns_server)

    def performNSLookup(self, hostname, dns_server = None):
        """
        Performs a NS lookup and returns an array containg all nameservers in
        the response.

        :hostname: is the hostname to perform the NS lookup on

        :dns_server: is the dns_server that should be used for the lookup as a
                     tuple of ip port (ex. ("127.0.0.1", 53))

                     if None, system dns settings will be used
        """
        return self.dnsLookup(hostname, 'NS', dns_server)

    def performSOALookup(self, hostname, dns_server = None):
        """
        Performs a SOA lookup and returns the response (name,serial).

        :hostname: is the hostname to perform the SOA lookup on
        :dns_server: is the dns_server that should be used for the lookup as a
                     tuple of ip port (ex. ("127.0.0.1", 53))

                     if None, system dns settings will be used
        """
        return self.dnsLookup(hostname,'SOA',dns_server)

    def dnsLookup(self, hostname, dns_type, dns_server = None):
        """
        Performs a DNS lookup and returns the response.

        :hostname: is the hostname to perform the DNS lookup on
        :dns_type: type of lookup 'NS'/'A'/'SOA'
        :dns_server: is the dns_server that should be used for the lookup as a
                     tuple of ip port (ex. ("127.0.0.1", 53))
        """
        types = {
            'NS': dns.NS,
            'A': dns.A,
            'SOA': dns.SOA,
            'PTR': dns.PTR
        }
        dnsType = types[dns_type]
        query = [dns.Query(hostname, dnsType, dns.IN)]
        def gotResponse(message):
            log.debug(dns_type + " Lookup successful")
            log.debug(str(message))

            if dns_server:
                msg = message.answers
            else:
                msg = message[0]

            answers = []
            addrs = []
            for answer in msg:
                addr = None
                if answer.type is dns.SOA:
                    addr = (answer.name.name,answer.payload.serial)
                elif answer.type in [dns.NS, dns.PTR, dns.CNAME]:
                    addr = answer.payload.name.name
                elif answer.type is dns.A:
                    addr = answer.payload.dottedQuad()
                else:
                    log.debug("Unidentified answer %s" % answer)
                addrs.append(addr)
                answers.append(representAnswer(answer))

            if dns_type == 'SOA':
                for authority in message.authority:
                    answers.append(representAnswer(authority))

            DNSTest.addToReport(self, query, resolver=dns_server,
                                query_type=dns_type, answers=answers)
            return addrs

        def gotError(failure):
            failure.trap(gaierror, TimeoutError)
            DNSTest.addToReport(self, query, resolver=dns_server,
                                query_type=dns_type, failure=failure)
            return failure

        if dns_server:
            resolver = Resolver(servers=[dns_server])
            d = resolver.queryUDP(query, timeout=self.queryTimeout)
        else:
            lookupFunction = {
                'NS': client.lookupNameservers,
                'SOA': client.lookupAuthority,
                'A': client.lookupAddress,
                'PTR': client.lookupPointer
            }
            d = lookupFunction[dns_type](hostname)

        d.addCallback(gotResponse)
        d.addErrback(gotError)
        return d

    def addToReport(self, query, resolver=None, query_type=None,
                    answers=None, failure=None):
        log.debug("Adding %s to report)" % query)
        result = {
            'resolver_hostname': None,
            'resolver_port': None
        }
        if resolver is not None and len(resolver) == 2:
            result['resolver_hostname'] = resolver[0]
            result['resolver_port'] = resolver[1]
        result['query_type'] = query_type
        result['hostname'] = str(query[0].name)
        result['failure'] = None
        if failure:
            result['failure'] = failureToString(failure)

        result['answers'] = []
        if answers:
            result['answers'] = answers

        self.report['queries'].append(result)



# -*- encoding: utf-8 -*-

__author__ = "Open Observatory of Network Interference"
__version__ = "1.6.0"
# This is the version number of resources to be downloaded
# when a release is made it should be aligned to __version__
__resources_version__ = "1.6.0"

__all__ = [
    'common',
    'deckgen',
    'nettest',
    'report',
    'resources',
    'templates',
    'utils'
]



import os
import re
import string
import StringIO
import subprocess
from distutils.spawn import find_executable
from distutils.version import LooseVersion

from twisted.internet import reactor, defer
from twisted.internet.endpoints import TCP4ClientEndpoint

from txtorcon import TorConfig, TorState, launch_tor, build_tor_connection
from txtorcon.util import find_tor_binary as tx_find_tor_binary

from ooni import constants
from ooni import errors
from ooni.utils import log
from ooni.settings import config

ONION_ADDRESS_REGEXP = re.compile("^((httpo|http|https)://)?"
                                  "[a-z0-9]{16}\.onion")

TBB_PT_PATHS = ("/Applications/TorBrowser.app/Contents/MacOS/Tor"
                "/PluggableTransports/",)

class TorVersion(LooseVersion):
    pass


class OBFSProxyVersion(LooseVersion):
    pass


def find_tor_binary():
    if config.advanced.tor_binary:
        return config.advanced.tor_binary
    return tx_find_tor_binary()


def executable_version(binary, strip=lambda x: x):
    if not binary:
        return None
    try:
        proc = subprocess.Popen((binary, '--version'),
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except OSError:
        pass
    else:
        stdout, _ = proc.communicate()
        if proc.poll() == 0 and stdout != '':
            version = stdout.strip()
            return LooseVersion(strip(version))
    return None


def tor_version():
    version = executable_version(find_tor_binary(),
                                 lambda x: x.split(' ')[2])
    return TorVersion(str(version))


def obfsproxy_version():
    version = executable_version(find_executable('obfsproxy'))
    return OBFSProxyVersion(str(version))


def transport_name(address):
    """
    If the address of the bridge starts with a valid c identifier then
    we consider it to be a bridge.
    Returns:
        The transport_name if it's a transport.
        None if it's not a obfsproxy bridge.
    """
    transport_name = address.split(' ')[0]
    transport_name_chars = string.ascii_letters + string.digits
    if all(c in transport_name_chars for c in transport_name):
        return transport_name
    return None


def is_onion_address(address):
    return ONION_ADDRESS_REGEXP.match(address) != None


def find_pt_executable(name):
    bin_loc = find_executable(name)
    if bin_loc:
        return bin_loc
    for path in TBB_PT_PATHS:
        bin_loc = os.path.join(path, name)
        if os.path.isfile(bin_loc):
            return bin_loc
    return None

tor_details = {
    'binary': find_tor_binary(),
    'version': tor_version()
}

obfsproxy_details = {
    'binary': find_executable('obfsproxy'),
    'version': obfsproxy_version()
}

transport_bin_name = { 'fte': 'fteproxy',
                       'scramblesuit': 'obfsproxy',
                       'obfs2': 'obfsproxy',
                       'obfs3': 'obfsproxy',
                       'obfs4': 'obfs4proxy' }

_pyobfsproxy_line = lambda transport, bin_loc, log_file: \
    "%s exec %s --log-min-severity info --log-file %s managed" % \
    (transport, bin_loc, log_file)

_transport_line_templates = {
    'fte': lambda bin_loc, log_file : \
        "fte exec %s --managed" % bin_loc,

    'scramblesuit': lambda bin_loc, log_file: \
        _pyobfsproxy_line('scramblesuit', bin_loc, log_file),

    'obfs2': lambda bin_loc, log_file: \
        _pyobfsproxy_line('obfs2', bin_loc, log_file),

    'obfs3': lambda bin_loc, log_file: \
        _pyobfsproxy_line('obfs3', bin_loc, log_file),

    'obfs4': lambda bin_loc, log_file: \
        "obfs4 exec %s --enableLogging=true --logLevel=INFO" % bin_loc,

}

class UnrecognizedTransport(Exception):
    pass
class UninstalledTransport(Exception):
    pass
class OutdatedObfsproxy(Exception):
    pass
class OutdatedTor(Exception):
    pass

def bridge_line(transport, log_file):
    bin_name = transport_bin_name.get(transport)
    if not bin_name:
        raise UnrecognizedTransport

    bin_loc = find_executable(bin_name)
    if not bin_loc:
        raise UninstalledTransport

    if OBFSProxyVersion('0.2') > obfsproxy_details['version']:
        raise OutdatedObfsproxy

    if (transport == 'scramblesuit' or \
            bin_name == 'obfs4proxy') and \
            TorVersion('0.2.5.1') > tor_details['version']:
        raise OutdatedTor

    if TorVersion('0.2.4.1') > tor_details['version']:
        raise OutdatedTor

    return _transport_line_templates[transport](bin_loc, log_file)

pt_config = {
    'meek': [
        {
            'executable': 'obfs4proxy',
            'minimum_version': '0.0.6',
            'version_parse': lambda x: x.split('-')[1],
            'client_transport_line': 'meek exec {bin_loc}'
        },
        {
            'executable': 'meek-client',
            'minimum_version': None,
            'client_transport_line': 'meek exec {bin_loc}'
        }
    ],

    'obfs4': [
        {
            'executable': 'obfs4proxy',
            'minimum_version': None,
            'client_transport_line': 'obfs4 exec {bin_loc}'
        }
    ]

}

def get_client_transport(transport):
    """

    :param transport:
    :return: client_transport_line
    """

    try:
        pts = pt_config[transport]
    except KeyError:
        raise UnrecognizedTransport

    for pt in pts:
        bin_loc = find_pt_executable(pt['executable'])
        if bin_loc is None:
            continue
        if pt['minimum_version'] is not None:
            pt_version = executable_version(bin_loc, pt['version_parse'])
            if (pt_version is None or
                        pt_version < LooseVersion(pt['minimum_version'])):
                continue
        return pt['client_transport_line'].format(bin_loc=bin_loc)

    raise UninstalledTransport


class TorLauncherWithRetries(object):
    def __init__(self, tor_config, timeout=config.tor.timeout):
        self.retry_with = ["obfs4", "meek"]
        self.started = defer.Deferred()
        self.tor_output = StringIO.StringIO()
        self.tor_config = tor_config
        if timeout is None:
            # XXX we will want to move setting the default inside of the
            # config object.
            timeout = 200
        self.timeout = timeout

    def _reset_tor_config(self):
        """
        This is used to reset the Tor configuration to before launch_tor
        modified it. This is in particular used to force the regeneration of the
        DataDirectory.
        """
        new_tor_config = TorConfig()
        for key in self.tor_config:
            if config.tor.data_dir is None and key == "DataDirectory":
                continue
            setattr(new_tor_config, key, getattr(self.tor_config, key))
        self.tor_config = new_tor_config

    def _progress_updates(self, prog, tag, summary):
        log.msg("%d%%: %s" % (prog, summary))

    @defer.inlineCallbacks
    def _state_complete(self, state):
        config.tor_state = state
        log.msg("Successfully bootstrapped Tor")
        log.debug("We now have the following circuits: ")
        for circuit in state.circuits.values():
            log.debug(" * %s" % circuit)

        socks_port = yield state.protocol.get_conf("SocksPort")
        control_port = yield state.protocol.get_conf("ControlPort")

        config.tor.socks_port = int(socks_port.values()[0])
        config.tor.control_port = int(control_port.values()[0])
        self.started.callback(state)

    def _setup_failed(self, failure):
        self.tor_output.seek(0)
        map(log.debug, self.tor_output.readlines())
        self.tor_output.seek(0)

        if len(self.retry_with) == 0:
            self.started.errback(errors.UnableToStartTor())
            return

        while len(self.retry_with) > 0:
            self._reset_tor_config()
            self.tor_config.UseBridges = 1
            transport = self.retry_with.pop(0)
            log.msg("Failed to start Tor. Retrying with {0}".format(transport))

            try:
                bridge_lines = getattr(constants,
                                       '{0}_BRIDGES'.format(transport).upper())
            except AttributeError:
                continue

            try:
                self.tor_config.ClientTransportPlugin = get_client_transport(transport)
            except UninstalledTransport:
                log.err("Pluggable transport {0} is not installed".format(
                    transport))
                continue
            except UnrecognizedTransport:
                log.err("Unrecognized transport type")
                continue

            self.tor_config.Bridge = bridge_lines
            self.launch()
            break

    def _setup_complete(self, proto):
        """
        Called when we read from stdout that Tor has reached 100%.
        """
        log.debug("Building a TorState")
        config.tor.protocol = proto
        state = TorState(proto.tor_protocol)
        state.post_bootstrap.addCallbacks(self._state_complete,
                                          self._setup_failed)

    def _launch_tor(self):
        return launch_tor(self.tor_config, reactor,
                          tor_binary=config.advanced.tor_binary,
                          progress_updates=self._progress_updates,
                          stdout=self.tor_output,
                          timeout=self.timeout,
                          stderr=self.tor_output)

    def launch(self):
        self._launched = self._launch_tor()
        self._launched.addCallbacks(self._setup_complete, self._setup_failed)
        return self.started


def start_tor(tor_config):
    tor_launcher = TorLauncherWithRetries(tor_config)
    return tor_launcher.launch()


@defer.inlineCallbacks
def connect_to_control_port():
    connection = TCP4ClientEndpoint(reactor, '127.0.0.1',
                                    config.tor.control_port)
    config.tor_state = yield build_tor_connection(connection)



# -*- encoding: utf-8 -*-
from twisted.python import usage

from ooni.utils import log
from ooni.utils import randomStr
from ooni.templates import tcpt

class UsageOptions(usage.Options):
    optParameters = [['backend', 'b', '127.0.0.1',
                        'The OONI backend that runs a TCP echo server'],
                    ['backendport', 'p', 80, 'Specify the port that the TCP echo server is running (should only be set for debugging)']]

class HTTPFilteringBypass(tcpt.TCPTest):
    name = "HTTPFilteringBypass"
    version = "0.1"
    authors = "xx"

    inputFile = ['file', 'f', None,
            'Specify a list of hostnames to use as inputs']

    usageOptions = UsageOptions
    requiredOptions = ['backend']
    requiresRoot = False
    requiresTor = False

    def setUp(self):
        self.port = int(self.localOptions['backendport'])
        self.address = self.localOptions['backend']
        self.report['tampering'] = None

    def check_for_manipulation(self, response, payload):
        log.debug("Checking if %s == %s" % (response, payload))
        if response != payload:
            self.report['tampering'] = True
        else:
            self.report['tampering'] = False

    def test_prepend_newline(self):
        payload = "\nGET / HTTP/1.1\n\r"
        payload += "Host: %s\n\r" % self.input

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_tab_trick(self):
        payload = "GET / HTTP/1.1\n\r"
        payload += "Host: %s\t\n\r" % self.input

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_subdomain_blocking(self):
        payload = "GET / HTTP/1.1\n\r"
        payload += "Host: %s\n\r" % randomStr(10) + '.' + self.input

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_fuzzy_domain_blocking(self):
        hostname_field = randomStr(10) + '.' + self.input + '.' + randomStr(10)
        payload = "GET / HTTP/1.1\n\r"
        payload += "Host: %s\n\r" % hostname_field

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_fuzzy_match_blocking(self):
        hostname_field = randomStr(10) + self.input + randomStr(10)
        payload = "GET / HTTP/1.1\n\r"
        payload += "Host: %s\n\r" % hostname_field

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d

    def test_normal_request(self):
        payload = "GET / HTTP/1.1\n\r"
        payload += "Host: %s\n\r" % self.input

        d = self.sendPayload(payload)
        d.addCallback(self.check_for_manipulation, payload)
        return d




from twisted.internet import defer, reactor
from ooni.errors import handleAllFailures
from ooni.templates import scapyt
from ooni.utils import log
from ooni.utils.txscapy import ParasiticTraceroute
from ooni.settings import config

from scapy.all import TCPerror, IPerror

class ParasiticTracerouteTest(scapyt.BaseScapyTest):
    name = "Parasitic Traceroute Test"
    description = "Injects duplicate TCP packets with varying TTL values by sniffing traffic"
    version = '0.1'

    samplePeriod = 40
    requiresTor = False

    def setUp(self):
        self.report['parasitic_traceroute'] = {}

    def test_parasitic_traceroute(self):
        self.pt = ParasiticTraceroute()
        log.debug("Starting ParasiticTraceroute for up to %d hosts at inject "
                "rate %d with %s" % (self.pt.numHosts, self.pt.rate, self.pt))
        config.scapyFactory.registerProtocol(self.pt)
        d = defer.Deferred()
        reactor.callLater(self.samplePeriod, d.callback, self)
        d.addCallback(self.addToReport)
        d.addErrback(handleAllFailures)
        return d

    def addToReport(self, result):
        log.debug("Stopping ParasiticTraceroute")
        self.pt.stopListening()
        self.report['received_packets'] = self.pt.received_packets

        for packet in self.pt.received_packets:
            k = (packet[IPerror].id, packet[TCPerror].sport, packet[TCPerror].dport, packet[TCPerror].seq)
            if k in self.pt.matched_packets:
                ttl = self.pt.matched_packets[k]['ttl']
            else:
                ttl = 'unknown'
            hop = (ttl, packet.src)
            path = 'hops_%s' % packet[IPerror].dst
            if path in self.report['parasitic_traceroute']:
               self.report['parasitic_traceroute'][path].append(hop)
            else:
               self.report['parasitic_traceroute'][path] = [hop]
        for p in self.report['parasitic_traceroute'].keys():
            self.report['parasitic_traceroute'][p].sort(key=lambda x: x[0])
                
        self.report['sent_packets'] = self.pt.sent_packets



from mock import patch, MagicMock

from ooni.settings import config
from ooni.director import Director
from ooni.tests.bases import ConfigTestCase

from ooni.nettest import NetTestLoader

from twisted.internet import defer
from twisted.trial import unittest

from txtorcon import TorControlProtocol

test_failing_twice = """
from twisted.internet import defer, reactor
from ooni.nettest import NetTestCase

class TestFailingTwice(NetTestCase):
    inputs = ["spam-{}".format(idx) for idx in range(50)]

    def setUp(self):
        self.summary[self.input] = self.summary.get(self.input, 0)

    def test_a(self):
        run_count = self.summary[self.input]
        delay = float(self.input.split("-")[1])/1000
        d = defer.Deferred()
        def callback():
            self.summary[self.input] += 1
            if run_count < 3:
                d.errback(Exception("Failing"))
            else:
                d.callback(self.summary[self.input])

        reactor.callLater(delay, callback)
        return d
"""

proto = MagicMock()
proto.tor_protocol = TorControlProtocol()

mock_TorState = MagicMock()
# We use the instance of mock_TorState so that the mock caching will
# return the same instance when TorState is created.
mts = mock_TorState()
mts.protocol.get_conf = lambda x: defer.succeed({'SocksPort': '4242'})
mts.post_bootstrap = defer.succeed(mts)

# Set the tor_protocol to be already fired
state = MagicMock()
proto.tor_protocol.post_bootstrap = defer.succeed(state)

mock_launch_tor = MagicMock()
mock_launch_tor.return_value = defer.succeed(proto)


class TestDirector(ConfigTestCase):
    def tearDown(self):
        super(TestDirector, self).tearDown()
        config.tor_state = None

    def test_get_net_tests(self):
        director = Director()
        nettests = director.getNetTests()
        assert 'http_requests' in nettests
        assert 'dns_consistency' in nettests
        assert 'http_header_field_manipulation' in nettests
        assert 'traceroute' in nettests

    @patch('ooni.utils.onion.TorState', mock_TorState)
    @patch('ooni.utils.onion.launch_tor', mock_launch_tor)
    def test_start_tor(self):
        @defer.inlineCallbacks
        def director_start_tor():
            director = Director()
            yield director.startTor()
            assert config.tor.socks_port == 4242
            assert config.tor.control_port == 4242

        return director_start_tor()

    def test_run_test_fails_twice(self):
        finished = defer.Deferred()

        def net_test_done(net_test):
            summary_items = net_test.summary.items()
            self.assertEqual(len(summary_items), 50)
            for input_name, run_count in summary_items:
                self.assertEqual(run_count, 3)
            finished.callback(None)

        net_test_loader = NetTestLoader(('spam','ham'))
        net_test_loader.loadNetTestString(test_failing_twice)
        director = Director()
        director.netTestDone = net_test_done
        director.startNetTest(net_test_loader, None, no_yamloo=True)
        return finished


class TestStartSniffing(unittest.TestCase):
    def setUp(self):
        self.director = Director()
        self.testDetails = {
            'test_name': 'foo',
            'test_start_time': '2016-01-01 12:34:56'
        }

        # Each NetTestCase has a name attribute
        class FooTestCase(object):
            name = 'foo'
        self.FooTestCase = FooTestCase

    def test_start_sniffing_once(self):
        with patch('ooni.settings.config.scapyFactory') as mock_scapy_factory:
            with patch('ooni.utils.txscapy.ScapySniffer') as mock_scapy_sniffer:
                self.director.startSniffing(self.testDetails)
                sniffer = mock_scapy_sniffer.return_value
                mock_scapy_factory.registerProtocol.assert_called_once_with(sniffer)

    def test_start_sniffing_twice(self):
        with patch('ooni.settings.config.scapyFactory') as mock_scapy_factory:
            with patch('ooni.utils.txscapy.ScapySniffer') as mock_scapy_sniffer:
                sniffer = mock_scapy_sniffer.return_value
                sniffer.pcapwriter.filename = 'foo1_filename'
                self.director.startSniffing(self.testDetails)
                self.assertEqual(len(self.director.sniffers), 1)

            self.testDetails = {
                'test_name': 'bar',
                'test_start_time': '2016-01-01 12:34:56'
            }
            with patch('ooni.utils.txscapy.ScapySniffer') as mock_scapy_sniffer:
                sniffer = mock_scapy_sniffer.return_value
                sniffer.pcapwriter.filename = 'foo2_filename'
                self.director.startSniffing(self.testDetails)
                self.assertEqual(len(self.director.sniffers), 2)

    def test_measurement_succeeded(self):
        with patch('ooni.settings.config.scapyFactory') as mock_scapy_factory:
            with patch('ooni.utils.txscapy.ScapySniffer') as mock_scapy_sniffer:
                self.director.startSniffing(self.testDetails)
                self.assertEqual(len(self.director.sniffers), 1)
                measurement = MagicMock()
                measurement.testInstance = self.FooTestCase()
                self.director.measurementSucceeded('awesome', measurement)
                self.assertEqual(len(self.director.sniffers), 0)
                sniffer = mock_scapy_sniffer.return_value
                mock_scapy_factory.unRegisterProtocol.assert_called_once_with(sniffer)




import os
from twisted.trial import unittest

from ooni.utils import log, generate_filename, net


class TestUtils(unittest.TestCase):
    def setUp(self):
        self.test_details = {
            'test_name': 'foo',
            'test_start_time': '2016-01-01 01:22:22'
        }
        self.extension = 'ext'
        self.prefix = 'prefix'
        self.basename = 'filename'
        self.filename = 'filename.txe'

    def test_log_encode(self):
        logmsgs = (
            (r"spam\x07\x08", "spam\a\b"),
            (r"spam\x07\x08", u"spam\a\b"),
            (r"ham\u237e", u"ham"+u"\u237e")
        )
        for encoded_logmsg, logmsg in logmsgs:
            self.assertEqual(log.log_encode(logmsg), encoded_logmsg)

    def test_generate_filename(self):
        filename = generate_filename(self.test_details)
        self.assertEqual(filename, 'foo-2016-01-01T012222Z')

    def test_generate_filename_with_extension(self):
        filename = generate_filename(self.test_details, extension=self.extension)
        self.assertEqual(filename, 'foo-2016-01-01T012222Z.ext')

    def test_generate_filename_with_prefix(self):
        filename = generate_filename(self.test_details, prefix=self.prefix)
        self.assertEqual(filename, 'prefix-foo-2016-01-01T012222Z')

    def test_generate_filename_with_extension_and_prefix(self):
        filename = generate_filename(self.test_details, prefix=self.prefix, extension=self.extension)
        self.assertEqual(filename, 'prefix-foo-2016-01-01T012222Z.ext')

    def test_get_addresses(self):
        addresses = net.getAddresses()
        assert isinstance(addresses, list)



# -*- encoding: utf-8 -*-
#
# This is a wrapper around the Netalyzer Java command line client
#
# :authors: Jacob Appelbaum <jacob@appelbaum.net>
#           Arturo "hellais" Filastò <art@fuffa.org>
# :licence: see LICENSE

import time
import os
import distutils.spawn

from twisted.python import usage
from twisted.internet import reactor, threads

from ooni.templates import process
from ooni.utils import log

class JavaNotInstalled(Exception):
    pass

class CouldNotFindNetalyzrCli(Exception):
    pass

class UsageOptions(usage.Options):
    optParameters = [
        ['clipath', 'p', None, 'Specify the path to NetalyzrCLI.jar (can be '
                               'downloaded from '
                               'http://netalyzr.icsi.berkeley.edu/NetalyzrCLI.jar).']
    ]

class NetalyzrWrapperTest(process.ProcessTest):
    name = "NetalyzrWrapper"
    description = "A wrapper around the Netalyzr java command line client."
    author = "Jacob Appelbaum <jacob@appelbaum.net>"

    requiredOptions = ['clipath']

    usageOptions = UsageOptions
    requiresRoot = False
    requiresTor = False

    timeout = 300

    def requirements(self):
        if not distutils.spawn.find_executable("java"):
            raise JavaNotInstalled("Java is not installed.")

    def setUp(self):
        if not os.path.exists(self.localOptions['clipath']):
            raise CouldNotFindNetalyzrCli("Could not find NetalyzrCLI.jar at {}".format(self.localOptions['clipath']))

        self.command = [
            distutils.spawn.find_executable("java"),
            "-jar",
            "{}".format(self.localOptions['clipath']),
            "-d"
        ]

    def test_run_netalyzr(self):
        """
        This test simply wraps netalyzr and runs it from command line
        """
        log.msg("Running NetalyzrWrapper (this will take some time, be patient)")
        log.debug("with command '%s'" % self.command)
        self.d = self.run(self.command, env=os.environ, usePTY=1)
        return self.d



import os
import re
import copy
import json
import types
import tempfile
import functools

from twisted.python import usage
from cyclone import web, escape

from ooni.reporter import YAMLReporter, OONIBReporter, collector_supported
from ooni import errors
from ooni.nettest import NetTestLoader
from ooni.settings import config

class InvalidInputFilename(Exception):
    pass

class FilenameExists(Exception):
    pass

def check_xsrf(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kw):
        xsrf_header = self.request.headers.get("X-XSRF-TOKEN")
        if self.xsrf_token != xsrf_header:
            raise web.HTTPError(403, "Invalid XSRF token.")
        return method(self, *args, **kw)
    return wrapper

class ORequestHandler(web.RequestHandler):
    serialize_lists = True
    xsrf_cookie_name = "XSRF-TOKEN"

    def write(self, chunk):
        """
        XXX This is a patch that can be removed once
        https://github.com/fiorix/cyclone/pull/92 makes it into a release.
        """
        if isinstance(chunk, types.ListType):
            chunk = escape.json_encode(chunk)
            self.set_header("Content-Type", "application/json")
        web.RequestHandler.write(self, chunk)

class Status(ORequestHandler):
    @check_xsrf
    def get(self):
        result = {'active_tests': oonidApplication.director.activeNetTests}
        self.write(result)

def list_inputs():
    input_list = []
    for filename in os.listdir(config.inputs_directory):
        input_list.append({'filename': filename})
    return input_list

class Inputs(ORequestHandler):
    """
    This handler is responsible for listing and adding new inputs.
    """

    @check_xsrf
    def get(self):
        """
        Obtain the list of currently installed inputs. Inputs are stored inside
        of $OONI_HOME/inputs/.
        """
        input_list = list_inputs()
        self.write(input_list)

    @check_xsrf
    def post(self):
        """
        Add a new input to the currently installed inputs.
        """
        input_file = self.request.files.get("file")[0]
        filename = input_file['filename']

        if not filename or not re.match('(\w.*\.\w.*).*', filename):
            raise InvalidInputFilename

        if os.path.exists(filename):
            raise FilenameExists

        content_type = input_file["content_type"]
        body = input_file["body"]

        fn = os.path.join(config.inputs_directory, filename)
        with open(os.path.abspath(fn), "w") as fp:
            fp.write(body)

class ListTests(ORequestHandler):

    @check_xsrf
    def get(self):
        test_list = copy.deepcopy(oonidApplication.director.netTests)
        for test_id in test_list.keys():
            test_list[test_id].pop('path')
        self.write(test_list)

def get_net_test_loader(test_options, test_file):
    """
    Args:
        test_options: (dict) containing as keys the option names.

        test_file: (string) the path to the test_file to be run.
    Returns:
        an instance of :class:`ooni.nettest.NetTestLoader` with the specified
        test_file and the specified options.
        """
    options = []
    for k, v in test_options.items():
        options.append('--'+k)
        options.append(v)

    net_test_loader = NetTestLoader(options,
            test_file=test_file)
    return net_test_loader

def get_reporters(net_test_loader):
    """
    Determines which reports are able to run and returns an instance of them.

    We always report to flat file via the :class:`ooni.reporters.YAMLReporter`
    and the :class:`ooni.reporters.OONIBReporter`.

    The later will be used only if we determine that Tor is running.

    Returns:
        a list of reporter instances
    """
    test_details = net_test_loader.testDetails
    reporters = []
    yaml_reporter = YAMLReporter(test_details, config.reports_directory)
    reporters.append(yaml_reporter)

    if config.reports.collector and collector_supported(config.reports.collector):
        oonib_reporter = OONIBReporter(test_details, config.reports.collector)
        reporters.append(oonib_reporter)
    return reporters

def write_temporary_input(content):
    """
    Creates a temporary file for the given content.

    Returns:
        the path to the temporary file.
    """
    fd, path = tempfile.mkstemp()
    with open(path, 'w') as f:
        f.write(content)
        f.close()
    print "This is the path %s" % path
    return fd, path

class StartTest(ORequestHandler):

    @check_xsrf
    def post(self, test_name):
        """
        Starts a test with the specified options.
        """
        test_file = oonidApplication.director.netTests[test_name]['path']
        test_options = json.loads(self.request.body)
        tmp_files = []
        if ('manual_input' in test_options):
            for option, content in test_options['manual_input'].items():
                fd, path = write_temporary_input(content)
                test_options[option] = path
                tmp_files.append((fd, path))
            test_options.pop('manual_input')

        net_test_loader = get_net_test_loader(test_options, test_file)
        try:
            net_test_loader.checkOptions()
            d = oonidApplication.director.startNetTest(net_test_loader,
                                                       get_reporters(net_test_loader))
            @d.addBoth
            def cleanup(result):
                for fd, path in tmp_files:
                    os.close(fd)
                    os.remove(path)

        except errors.MissingRequiredOption, option_name:
            self.write({'error':
                        'Missing required option: "%s"' % option_name})
        except usage.UsageError, e:
            self.write({'error':
                        'Error in parsing options'})
        except errors.InsufficientPrivileges:
            self.write({'error':
                        'Insufficient priviledges'})

class StopTest(ORequestHandler):

    @check_xsrf
    def delete(self, test_name):
        pass

def get_test_results(test_id):
    """
    Returns:
        a list of test dicts that correspond to the test results for the given
        test_id.
        The dict is made like so:
        {
            'name': The name of the report,
            'content': The content of the report
        }
    """
    test_results = []
    for test_result in os.listdir(config.reports_directory):
        if test_result.startswith('report-'+test_id):
            with open(os.path.join(config.reports_directory, test_result)) as f:
                test_content = ''.join(f.readlines())
            test_results.append({'name': test_result,
                                 'content': test_content})
    test_results.reverse()
    return test_results

class TestStatus(ORequestHandler):

    @check_xsrf
    def get(self, test_id):
        """
        Returns the requested test_id details and the stored results for such
        test.
        """
        try:
            test = copy.deepcopy(oonidApplication.director.netTests[test_id])
            test.pop('path')
            test['results'] = get_test_results(test_id)
            self.write(test)
        except KeyError:
            self.write({'error':
                        'Test with such ID not found!'})

config.read_config_file()
oonidAPI = [
    (r"/status", Status),
    (r"/inputs", Inputs),
    (r"/test", ListTests),
    (r"/test/(.*)/start", StartTest),
    (r"/test/(.*)/stop", StopTest),
    (r"/test/(.*)", TestStatus),
    (r"/(.*)", web.StaticFileHandler,
        {"path": os.path.join(config.data_directory, 'ui', 'app'),
         "default_filename": "index.html"})
]

oonidApplication = web.Application(oonidAPI, debug=True)




#
# This unittest is to verify that our usage of the twisted DNS resolver does
# not break with new versions of twisted.

from twisted.trial import unittest
from twisted.names import dns
from twisted.names.client import Resolver


class DNSTest(unittest.TestCase):
    def test_a_lookup_ooni_query(self):
        def done_query(message, *arg):
            answer = message.answers[0]
            self.assertEqual(answer.type, 1)

        dns_query = [dns.Query('ooni.nu', type=dns.A)]
        resolver = Resolver(servers=[('8.8.8.8', 53)])
        d = resolver.queryUDP(dns_query)
        d.addCallback(done_query)
        return d




from ooni.templates import httpt, dnst

from ooni.tests import is_internet_connected

from twisted.names import dns
from twisted.internet.error import DNSLookupError
from twisted.internet import reactor, defer, base
from twisted.trial import unittest

base.DelayedCall.debug = True

class TestHTTPT(unittest.TestCase):
    def setUp(self):
        from twisted.web.resource import Resource
        from twisted.web.server import Site

        class DummyResource(Resource):
            isLeaf = True

            def render_GET(self, request):
                return "%s" % request.method

        r = DummyResource()
        factory = Site(r)
        self.port = reactor.listenTCP(8880, factory)

    def tearDown(self):
        self.port.stopListening()

    @defer.inlineCallbacks
    def test_do_request(self):
        http_test = httpt.HTTPTest()
        http_test.localOptions['socksproxy'] = None
        http_test._setUp()
        response = yield http_test.doRequest('http://localhost:8880/')
        assert response.body == "GET"
        assert len(http_test.report['requests']) == 1
        assert 'request' in http_test.report['requests'][0]
        assert 'response' in http_test.report['requests'][0]

    @defer.inlineCallbacks
    def test_do_failing_request(self):
        http_test = httpt.HTTPTest()
        http_test.localOptions['socksproxy'] = None
        http_test._setUp()
        yield self.assertFailure(http_test.doRequest('http://invaliddomain/'), DNSLookupError)
        assert http_test.report['requests'][0]['failure'] == 'dns_lookup_error'

class TestDNST(unittest.TestCase):
    def setUp(self):
        if not is_internet_connected():
            self.skipTest("You must be connected to the internet to run this test")

    def test_represent_answer_a(self):
        a_record = dns.RRHeader(payload=dns.Record_A(address="1.1.1.1"),
                                type=dns.A)
        self.assertEqual(dnst.representAnswer(a_record),
                         {'ipv4': '1.1.1.1', 'answer_type': 'A'})

    def test_represent_answer_ptr(self):
        ptr_record = dns.RRHeader(payload=dns.Record_PTR(name="example.com"),
                                  type=dns.PTR)
        self.assertEqual(dnst.representAnswer(ptr_record),
                         {'hostname': 'example.com', 'answer_type': 'PTR'})

    def test_represent_answer_soa(self):
        ptr_record = dns.RRHeader(payload=dns.Record_SOA(mname='example.com',
                                                         rname='foo.example.com'),
                                  type=dns.SOA)
        represented_answer = {}
        represented_answer['ttl'] = None
        represented_answer['answer_type'] = 'SOA'
        represented_answer['hostname'] = 'example.com'
        represented_answer['responsible_name'] = 'foo.example.com'
        represented_answer['serial_number'] = 0
        represented_answer['refresh_interval'] = 0
        represented_answer['retry_interval'] = 0
        represented_answer['minimum_ttl'] = 0
        represented_answer['expiration_limit'] = 0
        self.assertEqual(dnst.representAnswer(ptr_record),
                         represented_answer)

    @defer.inlineCallbacks
    def test_perform_a_lookup(self):
        if not is_internet_connected():
            self.skipTest("You must be connected to the internet to run this test")
        dns_test = dnst.DNSTest()
        dns_test._setUp()
        result = yield dns_test.performALookup('example.com', dns_server=('8.8.8.8', 53))
        self.assertEqual(result, ['93.184.216.34'])
